/*start==============================================================================================================================================================================================
   Automatische Steuerung zum Maischen

   Überarbeitung "Bitter" (Lothar) und Borsti84 (Sebastian)

   V 26.08.2017 - Relaisversion und Funkversion für den Arduino-Mega, optional auch ohne Display/Encoder, Display
   und kabellosen(Funk)-Temperatursensor.


   Updates:
   Nutzung mit KBH2AiO
   Nutzung eines zweiten Temperatursensors für die Nachgussbereitung (wird automatisch erkannt)
   Verwendung von Temperatur Funksensoren (immer als Maischesensor)
   Ansteuerung einer 3. Funksteckdose wahlweise für den Brauerruf oder für die parallele Nachgussbereitung
   Gärführung (Heizen UND Kühlen)
   Einstellung der Piezo-Lautstärke
   Loggen im Hintergrund
   Kein Neustart nach dem Anlernen der Funksteckdosen
   Optimierung automatische Gärführung
   Einschaltkontrolle Nachgusssteckdose
   Rührwerk mit Buttondruck umschaltbar (Dauerbetrieb -> Intervallbetrieb) - wenn Geschwindigkeitsrelais deaktiviert
   Rührwerk läuft während des "Pulsens" weiter



   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
   Die Software wird unter der MIT - Lizenz veröffentlicht

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files ,
   to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   Hiermit wird unentgeltlich jeder Person, die eine Kopie der Software und der zugehörigen Dokumentationen erhält, die Erlaubnis erteilt,
   sie uneingeschränkt zu benutzen, inklusive und ohne Ausnahme dem Recht, sie zu verwenden, kopieren, ändern, fusionieren, verlegen, verbreiten, unterlizenzieren
   und/oder zu verkaufen, und Personen, die diese Software erhalten, diese Rechte zu geben, unter den folgenden Bedingungen:
   Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in allen Kopien oder Teilkopien der Software beizulegen.
   DIE SOFTWARE WIRD OHNE JEDE AUSDRÜCKLICHE ODER IMPLIZIERTE GARANTIE BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FÜR DEN VORGESEHENEN ODER
   EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTSVERLETZUNG, JEDOCH NICHT DARAUF BESCHRÄNKT. IN KEINEM FALL SIND DIE AUTOREN ODER COPYRIGHTINHABER FÜR JEGLICHEN
   SCHADEN ODER SONSTIGE ANSPRÜCHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFÜLLUNG EINES VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER
   SOFTWARE ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

  Über diese Version

    Alle Einstellungen sind mit dem Encoder vorzunehmen.
      - Encoder drehen = Werteverstellung
      - Encoder schnell drehen = schnelle Werteverstellung
      - kurz Drücken = Eingabebestätigung
      - Doppelclick innerhalb von 0,6 s = Setupmenue -> Zurück -> lfd. Programm
        So kann z.B. während des Maischens im Setup-Menü der Gradientenfaktor geändert werden. Während der Zeit im Setup-Menü stoppt der Zeitablauf nicht!
      - lang Drücken (> 4 Sekunden) = Abbruch und zurück zum Startmenü
      - bei Abbruch werden alle eingegebenen Werte beibehalten
        Wenn man z.B. einen Fehler bei der Eingabe macht, einfach lange drücken bis zum Abbruch. Dann die Eingaben durch drücken und drehen ggf. verändern.
      - während des automatischen Ablaufs sind durch Drehen des Encoders Änderungen an der Temperatur bzw. des Zeitablaufes möglich.
      - Kühlmodus/Heizmodus: Wenn Sicherung gegen Stromausfall dann: Kühlmodus aufrufen -> Temperatur eingeben -> durch langes drücken Modus verlassen!
        In der unteren Zeile erscheint "NOT-EIN". Dann den Kühlmodus wieder aufrufen. Nach einem Stromausfall wird nun automatisch der Kühlmodus mit der zuletzt
        eingestellten Temperatur aufgerufen. Wenn der Modus nun beendet wird, erscheint in der unteren Zeile "NOT-AUS". Das Gleiche gilt für den Modus "Heizen".
      - Im Setup kann auch eine Notfallversion aktiviert werden. Der Notfall bedeutet "Stromausfall". Alle Werte werden anschließend übernommen und das Programm
        startet an der Stelle, an der der Stromausfall geschah.


    Regler
    Die Steuerung der Heizung erfolgt über eine Gradientenregelung. Die Steigungsgeschwindigkeit des Temperaturanstiegs dT/dt bestimmt die Ein- und Ausschaltzeitpunkte. Sinnvoll für
    Einkocher ist hier ein Gradientenfaktor von "1-1.2". Die Gärführung funktioniert über einen Zweipunktregler (Hysterese +/-0,5K)

    Sensorfehler
    Anzeige -127  => VCC (Versorgungsspannung des Sensors fehlt
    Anzeige 85.00   => Byte interner Sensorfehler ggf. Leitung zu lang
    Anzeige 0.00  => Datenleitung oder GND fehlt


    Schaltvorgänge des Ein- und Ausschalten sind immer nach einer einstellbaren Wartezeit möglich, dadurch soll stetiges Schalten verhindert werden.Für mechanische Relais > 5s. Für SSR´s kann der Wert auch "0" sein.

    Die Kühl-/Heizfunktion verwendet auch den Gradientenregler, jedoch kommt hier noch die Einschaltverzögerung für den Kompressor hinzu. Wird die eingestellte Tempertur erreicht und der Kompressor ausschaltet, dann
    dauert es mindestens die eingestelle Zeit bis der Kompressor wieder einschaltet (für einen Kühlschrank etwa 5 min. und für Peltier-Elemente 0 min.)
    Der Rührer/Malzrohrpumpe kann auf Dauerbetrieb oder Intervallbetrieb eingestellt werden.Die Einschalt- und Ausschaltsequenzen sind einstellbar. Im Setup kann von Rührer auf Malzrohrpumpe umgeschaltet werden, dann kann auch während der
    Heizphase Intervallbetrieb gewählt werden.
    Das Kochen des Suds kann erst nach einer vorgegebenen Temperaturschwelle erfolgen (sinnvoll 98°C). Dann ertönt der Brauerruf der quittiert werden muss. Sollte dies nicht erfolgen, schaltet der Kessel nach 1 min. ab.
    Dies soll das Überkochen verhinden. Erst danach kann die Zeitzählung aktiviert werden.
    Es können max. 6 Hopfengaben festgelegt werden. Erinnert wird durch den Brauerruf.


    Die Funktionen die auf den Vergleich Sollwert mit Istwert
    beruhen werden immer erst nach mehreren Messungen ausgeführt.

    Anschlüsse:
    PIN 2 : Encoder => Pullup geschaltet => Encoderzuleitung mit GND versorgen
    PIN 3 : Encoder => Pullup geschaltet => Encoderzuleitung mit GND versorgen
    PIN 4 : Taster von Encoder => Pullup geschaltet => Zuleitung mit GND versorgen
    PIN 5 : DS1820 ONE_WIRE_BUS Sensor => 4,7k zw. 5V und PIN5
    PIN 6 : Reset ESP8266-01
    PIN 7 : Heizung Nullleiter
    PIN 8 : Heizung Phase
    PIN 9 : Geschwindigkeitsumschaltung
    PIN 10: Ruf (Summer)
    PIN 11: Rührwerk Nullleiter
    PIN 12: Rührwerk Phase
    PIN 18: Funkempfänger (INT 5)
    PIN 19: Funksender
    PIN 20: Anschluss SDA vom Display
    PIN 21: Anschluss SCL vom Display

    PIN A0: Würzeumpensensor (Schwimmschalter)
    PIN A1: Würzepumpenrelais
    PIN A4: Nachgussheizung Phase
    PIN A5: Nachgussheizung Nullleiter




  Die folgenden Definitionen und Einstellungen beeinflussen das Verhalten der AiO. Diese Einstellungen können
  nicht im Setup verändert werden.
*/

//+++++++++++++++++++++++++++DEBUGGER++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//#define dbg  //Ausgabe von Schlüsselwerten über die serielle Schnittstelle ->Für Debuggen auskommentieren
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//++++++++++++++++++++++++++Versionierung++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Achtung! - es darf bei der Verwendung eines Funksensors zusätzlich nur ein kabelgebundener Sensor angeschlossen sein.
//#define Tempsensor433 //Auswahl der Versionen - mit FunkTempsensor oder ohne (einkommentiert = ohne Funksensor)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//++++++++++++++++++++++++++Display und Encoder entfall++++++++++++++++++++++++++++++++++++++++++++++++++++

//Falls die AiO ohne Display und Encoder betrieben wird
bool MitDisplayEncoder = true;  //diesen Wert auf "false" ändern, wenn kein Display/Encoder ahgeschlossen ist.
//Dadurch wird WLan eingeschaltet(kann über KBH2AiO nicht ausgeschaltet werden)

bool wlan_steuerung = false;  //Vorgabe falls mit Display / ohne Display wird dieser Wert automatisch auf true gesetzt (diesen Wert hier nicht ändern!)
byte kbh2aio = 0;             //Hilfsvariable
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//++++++++++++++++++++++++++Geschwindigkeitsumschaltung Rührmotor?+++++++++++++++++++++++++++++++++++++++++
//Anschluss wird im Hobbybrauerwiki beschrieben:
//(http://hobbybrauer.de/forum/wiki/doku.php/brauen_mit_arduino_all_in_one_aio#aufbau_der_aio-uno)

//#define geschwindigkeitsrelais //Geschwindigkeitsrelais für den Rührer verbaut?
//während der Rasten ist durch Drücken des Encoders die Geschwindigkeit umschaltbar
//sonst der Rührer über den Button manuell ein- und ausschaltbar

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++Einschaltverzögerung Nachgusstopf+++++++++++++++++++++++++++++++++++++++++++++++++

#define esv_nachguss 2000  //Einschaltverzögerung Nachgussheizung in den Rasten in ms
//Sicherheitsschwelle damit nicht beide Kessel gleichzeitig eingeschaltet sind

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++Gärhysterese++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float gaerhysterese = 0.3;  //0,5K über/unter dem Zielwert - diesen Wert bitte nicht kleiner als 0,1K (testen!)
//Eine zu kleine Hysterese führt zu einer konkurrierenden Regelung im Übergangsbereich!
//Bei "normalen" Kühlschränken auch 0,1K möglich!

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//++++++++++++++++++++++++++Verwendungsart der 3. Funksteckdose (Variabel)+++++++++++++++++++++++++++++++++
bool funkruf = true;      //3. Steckdose als Funkruf-Weiterleitung (Vorgabe)
//kann im Setup auch als Nachgussdose konfiguriert werden!

//+++++++++++++++++++++++Rührwerk beim Einmaischen aktiv?+++++++++++++++++++++++++++++++++++++++++++++++++

bool ruehrer_beim_einmaischen = true; //Mischer beim Einmaischen aktiv
//Kann im Setup geändert werden, z.B wenn der Mischer eine Malzrohrpumpe ist!

//+++++++++++++++++++++++Zykluszeiten Daten senden +++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define PCZyklus  400   // Zykluszeit in ms für Daten zum PC
#define AIOZyklus  200  // Zykluszeit in ms für Daten vom PC zur AIO

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/* Ab hier die Werte bitte nicht mehr ändern!------------------------------------------------------------
*/

//Für die Ansteuerung von Heizung, Rührwerk und Nachguss können insgesamt 3/6 Relais für eine allpolige
//Ansteuerung angeschlossen werden. Natürlich kann man jeweils auch nur ein Relais zum Schalten verwenden.
//Aus Sicherheitsgründen habe ich aber die Möglichkeit der allpoligen Schaltung eingebaut!

//Ausgänge an DigitalPIN ----------------------------------------------------
#define ONE_WIRE_BUS 5                                  //Temperatursensoren - Hauptsensor/sep. Nachgusskessel
//---------------------------------------------------------------------------
#define HeizungR1  8                                    //Zuordnung Heizung Phase Relais1
#define HeizungR2  7                                    //Zuordnung Heizung Nullleiter Relais2

#ifdef geschwindigkeitsrelais
#define Speedrelais 9
#endif                                                  //Geschwindigkeitsumschaltung Rührerrelais
#define RufR  10                                        //Zuordnung Braumeisterruf
#define UmwaelzR2   11                                  //Zuordnung Umwaelzer (Rührer/Pumpe) Nullleiter Relais 2
#define UmwaelzR1   12                                  //Zuordnung Umwaelzer (Rührer/Pumpe) Phase Relais 1

//Ausgänge am AnalogPIN------------------------------------------------------
#define fuellstand_sensor A0                            //Steuerung der Würzepumpe
#define fuellstand_pumpe A1

//Nachgussrelais nur MIT 2. Temperatursensor möglich!
//Ist eine Funkdose für die Ansteuerung des Nachgusskessels konfiguriert, kann der dort eingebaute Sensor die Steuerung übernehmen!
//Ein 2. Temperatursensor ist dann NICHT notwendig!
#define NachgussR1  A4                                  //Zuordnung Nachgussrelais Phase Relais1 
#define NachgussR2  A5                                  //Zuordnung Nachgussrelais Nullleiter Relais2

//---------------------------------------------------------------------------


//ESP8266 -------------------------------------------------------------------
#define reset8266   6                                   //Resetpin ESP8266-1

//433MHz Sender/Empfänger----------------------------------------------------
#define sender 19                                       //Sender an Anschluss 19
#define empfaenger_int 5                                //477 MHz Empfänger an Interrupt 5 (Pin #18)
#define empfaengerPin 18                                //Digitalpin des Empfaengers (Temperatur Funksensor)
//---------------------------------------------------------------------------

/*

    Speicheraufteilung EEPROM ----------------------------------------------
    Falls die EEPROM-Adressen geändert werden, bitte auf den benötigten Speicherplatz achten
    Integer benötigt 2 Byte Speicherplatz
*/
#define notfallmodus 0              //Byte - Flag für Notfallmodus Maischen/Kochen
#define rastnr 1                    //Byte - Letzte Rast vor Stromausfall
#define rasttemp_aktuell 2          //Byte - aktuelle Rasttemperatur vor Stromausfall
#define rastzeit_aktuell 3          //Byte - aktuelle Rastzeit vor Stromausfall
#define rastzeit_1 4                //Byte - Speicher für eingestellte Rastzeitwerte
#define rasttemp_1 5                //Byte - Speicher für eingestellte Rasttemperaturen
#define rastzeit_2 6                //Byte
#define rasttemp_2 7                //Byte
#define rastzeit_3 8                //Byte
#define rasttemp_3 9                //Byte
#define rastzeit_4 10               //Byte
#define rasttemp_4 11               //Byte
#define rastzeit_5 12               //Byte
#define rasttemp_5 13               //Byte
#define rastzeit_6 14               //Byte
#define rasttemp_6 15               //Byte
#define MaischTemp 16               //Byte - Einmaischtemperatur
#define zeit_h 17                   //Byte - gespeicherte Zeit h,m,s
#define zeit_m 18                   //Byte
#define zeit_s 19                   //Byte
#define Rasten 20                   //Byte
#define rastmodus 21                //Integer - als long gespeichert - 4 Byte Platz lassen
#define notfall 25                  //Byte - Setupvariable
#define udauer 26                   //Byte - Rührer/Pumpe Dauerbetrieb ?
#define umwaelzer_ie 27             //Byte - Rührer/Pumpe Intervall EIN
#define umwaelzer_ia 28             //Byte - Rührer/Pumpe Intervall AUS
#define Kochschwelle 29             //Byte
#define logging 30                  //Byte - Setupvariable Logging EIN/AUS
#define kuehlverz 31                //Integer - als long gespeichert - 4 Byte Platz lassn - Einschaltverzögerung Kühlen
#define wein_h 35                   //Integer - als long gespeichert - 4 Byte Platz lassen - Einschaltverzögerung Heizen
#define graf_h 39                   //Integer - als long gespeichert - 4 Byte Platz lassen - Gradientenfaktor Heizen
#define graf_k 43                   //Integer - als long gespeichert - 4 Byte Platz lassen - Gradientenfaktor Kühlen
#define k_aktiv 47                  //Byte - Kühlen im Notfallmodus
#define g_kuehltemp 48              //Byte - zuletzt eingestellte Kühltemperatur
#define hopfengabe1 49              //Byte - Hopfengaben Notfallspeicher
#define hopfengabe2 50              //Byte
#define hopfengabe3 51              //Byte
#define hopfengabe4 52              //Byte
#define hopfengabe5 53              //Byte
#define hopfengabe6 54              //Byte
#define Kochzeit 55                 //Byte - Kochzeit Notfallspeicher
#define Wlan 56                     //Byte - WLAN Ein/Aus
#define Heizung_EIN 57              //LongInt Speicheradressen Steckdosencode
#define Heizung_AUS 61              //LongInt Speicheradressen Steckdosencode
#define Umwaelz_EIN 65              //LongInt Speicheradressen Steckdosencode
#define Umwaelz_AUS 69              //LongInt Speicheradressen Steckdosencode
#define Nachguss_EIN  73            //LongInt Speicheradressen Steckdosencode-auch für Funkrufversion
#define Nachguss_AUS 77             //LongInt Speicheradressen Steckdosencode-auch für Funkrufversion
#define isozeit_h 81                //Byte Stundenwert Nachisomerisierung
#define isozeit_m 82                //Byte Minutenwert Nachisomerisierung
#define isozeit_s 83                //Byte Sekundenwert Nachisomerisierung
#define niso 84                     //Byte Dauer Nachisomerisierung
#define loggerwahl 85               //BYTE Loggerprogramm auswählen
#define umwaelzwahl 86              //BYTE Rührwerk oder Malzrohrpumpe ?
#define dritte_funksteckdose 87     //BYTE 3. Dose als Funkruf oder als Nachgussschalter
#define heiz_kuehl 88               //Byte - Modus Kühlen/Heizen als Wahl
#define ruepumeinmaisch 89          //Byte - Pumpe/Rührer während des Einmaischens aktiv?
#define sound 90                    //Byte - Soundeinstellung für den Buzzer (Piezosummer)

//Modi der Aio --------------------------------------------------------------------------------------------------------

#define HAUPTMENUE 0
#define KUEHLEN 1
#define NACHGUSS 2
#define WUERZEPUMPE 3
#define MPUMPE 4
#define SETUPMENUE 5
#define BRAUMENUE 6
#define OPTIONENMENUE 7
#define SOUNDEINSTELLUNG 8
#define EXTERNMENUE 51
#define GRADIENTENFAKTOR_HEIZEN 11
#define EINSCHALTVERZOEGERUNG_HEIZRELAIS 12
#define GRADIENTENFAKTOR_KUEHLEN 13
#define WARTEZEIT_KUEHLEN 14
#define RUEPUMAKTIV 15
#define RASTANZAHL 19
#define EINMAISCHTEMPERATUR 20
#define RASTTEMPERATUREN 21
#define RASTZEITEN 22
#define AUSWAHL_UMWAELZER 23
#define UMWAELZER_DAUERLAUF 233
#define UMWAELZER_INTERVALL_EIN 234
#define UMWAELZER_INTERVALL_AUS 235
#define DRITTE_FUNKSTECKDOSE 236
#define STECKDOSENCODE 237
#define FRAGESTECKDOSENCODEEINGEBEN 238
#define KOCHSCHWELLE 239
#define WECHSEL_TEMP_ZEIT 24
#define CONNECT_WIFI 242
#define WIFI_EIN_AUS 243
#define ABMAISCHTEMPERATUR 25
#define MAISCHEN_START 26
#define EINMAISCHEN 27
#define TEMPERATURAUTOMATIK 28
#define ZEITAUTOMATIK 29
#define ENDTEMPERATURAUTOMATIK 30
#define RUFALARM 31
#define BRAUMEISTERRUFALARM 32
#define ANZAHL_HOPFENGABEN 37
#define NACHISOZEIT 38
#define HOPFENGABE 39
#define KOCHZEIT 40
#define STARTKOCHEN 41
#define KOCHEN 42
#define NACHISOMERISIERUNG 43
#define LOGGING_EIN_AUS 44
#define LOGWAHL 45
#define NOTFALLMODUS 46
#define WLANSTEUERUNG_AKTIV 48
#define ABBRUCH 80
#define GAERFUEHRUNG 245
#define GAERSTARTTEMPERATUR 246
#define GAERTEMPERATUREN 247
#define GAERZEITEN 248
#define WECHSEL_GAER_TEMP_ZEIT 249
#define ANSTELLTEMPERATUR 250
#define GAERZEITAUTOMATIK 251
#define ANLAUFGAERSTUFEN 252

//----------------------------------------------------------------------------------------------------------------------

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*Relaislogik festlegen
   ACHTUNG!!!!!!!
   bei Verwendung der China Relais-Platinen muss die Logik immer "AUS = HIGH" sein. Alle Geräte müssen dann am Schließer der
   Relais angeschlossen werden! Ansonsten liegt im ausgeschalteten Zustand Spannung am Ausgang!
   Schema im Wiki!
*/

/*Heizungsrelais +++++++++++++++++++
  //
  //Ist H_AUS als HIGH definiert, dann schaltet das Relais die Heizung im ANGEZOGENEN ZUSTAND AUS (Voreinstellung für Relaisplatinen)
  //Ist H_EIN als HIGH definiert, dann schaltet das Relais die Heizung im ANGEZOGENEN ZUSTAND EIN
  //Ist R_AUS als HIGH definiert, dann schaltet das Relais den Rührer im ANGEZOGENEN ZUSTAND AUS
  //Ist R_EIN als HIGH definiert, dann schaltet das Relais den Rührer im ANGEZOGENEN ZUSTAND EIN
  //
  //Die gewünschte Logik auskommentieren und die jeweils andere einkommentieren



  //Heizungsrelais +++++++++++++++++
*/

String hinweis;    //Anzeige im Startdisplay, welche Logik aktiv ist!

#define H_AUS LOW
#define H_EIN HIGH

//#define H_AUS HIGH                  //China-Relais-Platine
//#define H_EIN LOW

//Nachgusrelais +++++++++++++++++

#define NG_AUS LOW                   //wenn zusätzlich ein Nachgussrelais verbaut ist!
#define NG_EIN HIGH

//#define NG_AUS HIGH
//#define NG_EIN LOW

//Mischerrelais +++++++++++++++++++

#define R_AUS LOW
#define R_EIN HIGH

//#define R_AUS HIGH
//#define R_EIN LOW

//Rufrelais +++++++++++++++++++

#define B_EIN LOW
#define B_AUS HIGH

//Würzepumpe, nicht Malzrohrpumpe+++++++++++++++++++

#define WUERZEPUMPE_AUS LOW         //wenn ein Pumpenrelais verbaut ist
#define WUERZEPUMPE_EIN HIGH

//#define WUERZEPUMPE_AUS HIGH
//#define WUERZEPUMPE_EIN LOW

//welches Signal kommt vom Schwimmschalter? (kann an den meisten Schwimmschaltern konfiguriert werden!)
//Folgende Zuordnung wenn der Schwimmschalter schliesst

#define Sensor_AUS 1
#define Sensor_EIN 0

//Folgende Zuordnung wenn der Schwimmschalter öffnet

//#define Sensor_AUS 0
//#define Sensor_EIN 1

//---------------------------------------------------------------------------

//Speichervariable Steckdosencode -------------------------------------------

unsigned long funk_heizung_ein;
unsigned long funk_heizung_aus;

unsigned long funk_umwaelzer_ein;
unsigned long funk_umwaelzer_aus;

unsigned long funk_variabel_ein;
unsigned long funk_variabel_aus;

bool eingeschaltet = false;   //für Funk-Heizung EIN

byte adr = 1;  //Adresse im EPROM

#define RESEND_SNIFFED_VALUES 10             //jeden Funkbefehl 10 mal wiederholen
unsigned long value;

unsigned long heizungInterval1 = 0;          //Zeitstartwert um den den Zustand der Funksteckdosen überprüfen (aktuell im Sketch 5s!)
unsigned long heizungInterval2 = 0;          //dadurch wird versehentliches manuelles Schalten der Dosen verhindert!

//---------------------------------------------------------------------------

//Variablen Textblink für das LCD-Display -----------------------------------

unsigned long aktuelleMillis, alteMillis;
int blinkintervall = 1000;  //ms
bool textsichtbar = true;  //Blinken der Startabfrage für das Maischen

//LCD Initialisierung -------------------------------------------------------

#include <LiquidCrystal_I2C.h>

//LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Alternative I2C-Adresse
// falls 0x27 nicht funktioniert!
// Sollten beide Adressen nicht funktionieren,
// dann die Adresse mit dem I2C-Scanner (im Wiki) herausfinden!
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // aktuelle I2C-Adresse

//---------------------------------------------------------------------------

//433 MHz Funksender---------------------------------------------------------
#include <RCSwitch.h>
RCSwitch mySwitch = RCSwitch();

//Temperaturempfang über 433 MHz Sender -------------------------------------
//Schaltplan und Aufbau im Wiki!

#ifdef Tempsensor433
#include <VirtualWire.h>
#endif

//Click Encoder--------------------------------------------------------------
//die Bibliothek TimerOne ist nicht kompatibel mit VirtualWire, da beide
//den gleichen Interrupt verwenden. Daher kommt hier dann TimerThree zum
//Einsatz.

#include <ClickEncoder.h>
#ifdef Tempsensor433
#include <TimerThree.h>
#else
#include <TimerOne.h>
#endif

//Encoder Initialisierung ---------------------------------------------------

//Encoder drehen
#define pinA 2  //können auch beliebige andere Anschlüsse sein!
#define pinB 3  //tauschen der Werte für Drehumkehr

//#define ENC_DECODER (1 << 2)   //auskommentieren, wenn Probleme mit Encoder

#define ENC_HALFSTEP 1         //auskommentieren, wenn Probleme mit Encoder

#define stepsPerNotch 4          // Steps bis zum nächsten Schritt

// Encoder - Taster drücken
#define taster 4     // Taster am Pin 4 für Bestätigung und Abbruch

ClickEncoder *encoder;

void timerIsr() {
  encoder->service();
}

int last;         //letzter Wert

bool ButtonPressed = false;  //Encoder Werte
//---------------------------------------------------------------------------


//Temperatursensor DS1820 Initialisierung -----------------------------------
#include <OneWire.h>
#include <DallasTemperature.h>

// OneWire-Referenz
OneWire oneWire(ONE_WIRE_BUS);

// DS18B20 initialisieren
DallasTemperature sensors(&oneWire);

// Array für die Deviceadressen
DeviceAddress insideThermometer;

float sensorwert;                   //eingelesener Istwert (alle außer sep. Nachgussbereitung
float isttemp = 20;                 //Vorgabe 20 damit Sensorfehler am Anfang nicht anspricht
float nachgusssensor;               //2er Sensor für die Nachgussbereitung
byte nachgusssollwert = 78;         //Vorgabewert Nachgusstemperatur
byte anzahlTempsensoren;            //ist der Nachgusssensor angeschlossen?
byte nachgussheizung = 0;           //Merker ob Nachgussheizung EIN- bzw. Ausgeschaltet
bool wechsel = false;               //Wechsel zwischen Heizungs- und Nachgussrelais merken
bool funksensor = false;            //Vorgabe - nicht ändern!
//---------------------------------------------------------------------------

//Zeitmessung----------------------------------------------------------------
#include <TimeLib.h>
//---------------------------------------------------------------------------

//EEPROM für Voreinstellungen------------------------------------------------
#include <EEPROM.h>
//---------------------------------------------------------------------------

// Messergebnisse -----------------------------------------------------------
byte val = 0;        // Variable um Messergebnis zu speichern
//---------------------------------------------------------------------------

#define EIN 1
#define AUS 0


//Allgemein Initialisierung--------------------------------------------------
byte anfang = 0;                    //Start des jeweiligen Modus
float altsekunden;                  //Hilfsvariable Zeitmessung
byte regelung = AUS;                //Regelung aktiv/inaktiv - gilt nicht für die Gärführung
byte heizung = AUS;                 //Heizung EIN/AUS
byte sensorfehler = 0;              //Temperaturfühler defekt oder fehlt

bool sollwerterreicht = false;      //eingestellte Temperatur erreicht/nicht erreicht
bool nachguss_sollwerterreicht = false;  //dito für den Nachguss während des Maischens
long kwarten = 0;                   //Wartezeit kühlen
int modus = HAUPTMENUE;             //Programmmodus - 0=Hauptmenü
int sondermodus = 0;                //Sonderbehandlung im Brauerruf (Einmaischen und Ruflänge)
int rufmodus = 0;                   //merkt sich den Modus nach Brauerruf
float rufsignalzeit = 0;            //so lange dauert der Ruf - wird im Programm gesetzt
bool nachgussruf = false;           //Signal wenn Nachgusstemp erreicht
byte x = 1;                         //aktuelle Rast Nummer
byte altx;                          //Merker für letzte Rast
byte h = 0;                         //aktuelle Hopfengabe
byte y = 1;                         //Übergabewert von x für Braumeisterruf
byte n = 0;                         //Counter Messungserhöhung zur Fehlervermeidung
byte pause = 0;                     //Counter für Ruftonanzahl

byte sekunden = 0;                  //Zeitzählung
byte minuten = 0;                   //Zeitzählung
byte minutenwert = 0;               //Zeitzählung
byte stunden = 0;                   //Zeitzählung

byte blinker;                       //Hilfsvariable blinken;
byte abbruchkochen;                 //Zeitzählung bis Kochabbruch


//Vorgabewerte zur ersten Einstellung-------------------------------------------
byte sollwert = 20;                 //Sollwertvorgabe für Anzeige
byte maischtemp = 45;               //Vorgabe Einmasichtemperatur
byte rasten = 3;                    //Vorgabe Anzahl Rasten
byte rastTemp[] = {
  0, 55, 64, 72, 72, 72, 72, 72
};                                  //Rasttemperatur Werte
byte rastZeit[] = {
  0, 15, 40, 25, 20, 20, 20, 20
};                                  //Rastzeit Werte
byte braumeister[] = {
  0, 0, 0, 0, 0, 0
};                                  //Braumeisterruf Standard ist AUS
byte endtemp = 78;                  //Vorgabewert Endtemperatur
byte kochzeit = 90;                 // 90 min. Kochen

//Parameter zur Gärführung -------------------------------------------------------
//Startvorgaben

byte gaerstart = 8;  //°C
byte gaerrasten = 3;  //Anzahl Gärrasten
byte gaerRastTemp[] = {0, 8, 10, 16, 16, 16}; //°C
byte gaerRastZeit[] = {0, 24, 72, 254, 254, 254}; //in Stunden!
byte gaerregelung = AUS;  //wenn innerhalb der Solltemperatur +/- Hysterese

/* Hopfengaben----------------------------------------------------------------
  "maxrasten" kann geändert werden. die Vorgabewerte in den Klammern müssen nicht, können
  aber angepasst werden. "hanz" ist die Voreinstellung für die Auswahl der Rastanzahl.
*/

byte const maxgaben = 6;;
byte hopfengabe[maxgaben] = {90, 20, 10, 0, 0, 0};  //Zeiten für die Hopfengaben 90 min. können vorgegeben werden
byte hanz = 3;                      //Vorgabewert Anzahl Hopfengaben
bool rezept_eingelesen = false;

//----------------------------------------------------------------------------
byte nachisomerisierung = 20;  //Vorgabe 20 min. Wartezeit nach Kochende
byte isoTemperatur = 80;       //Temperaturgrenze für Nachisomerisierung
byte nachiso_h;                //Hilfsvariable zur Zeitsetzung nach Stromausfall
byte nachiso_m;
byte nachiso_s;
//----------------------------------------------------------------------------

//Rührwerk/Malzrohrpumpe------------------------------------------------------

char ja_nein[2] = {'N', 'J'};     //Antworten Dauerbetrieb Ja/nein
byte intervalEin = 60;            //Voreinstellung 60s EIN wenn noch kein Wert im EEPROM
byte intervalAus = 60;            //dann 60s AUS
char dauerbetrieb = 'N';          //Voreinstellung Intervallbetrieb
byte altdrehen;                   //alten Wert des Drehencoders merken
byte drehen;                      //aktueller Wert des Drehgebers
byte tempdrehen;                  //Merker für aktuellen Wert von "drehen" (für Doppelclick)
bool firstinterval = true;        //das erste Rührerintervall startet direkt nach Ende "Heizen"
long intervalstart;               //Intervallzyklus startet
byte rwerk = 0;                   //Hilfsvariable für Rührwekssteuerung (speichert den alten Zustand)
bool ruehrer = true;              //Vorgabe Rührer aktiv
bool pumpe = false;               //Pumpe alternativ
byte umwaelzen;                   //Hilfsvariable Umwälzerwahl
byte wahl;                        //Malzrohrpumpe oder Rührer
bool highspeed = false;           //Umschaltrelais verbaut?
bool button = false;              //Einschaltmöglichkeit des Rührers durch Buttondruck im Maischeprozess
#define LOW_SPEED 0               //Motor (Scheibenwischermotor langsam oder schnell
#define HIGH_SPEED 1

//Kochen---------------------------------------------------------------------
bool schwelle_erreicht = false;   //Kochschwelle erreicht, danach Wartezeit bis Abbruch

byte kschwelle = 98;              //Voreinstellung - ab hier ertönt der Brauerruf


unsigned long kzeitverz = 0;      //Wartezeit Einschalten Kompressor NICHT ÄNDERN
long esvheizen = 0;               //Einschaltverzögerung für Heizung - kann im Setup geändert werden
long esvpumpen = 30000;           //30s Einschaltverzögerung Pumpe - kann im Setup geändert werden
bool first = true;                //erster Anlauf ohne Zeitverzögerung
bool nachgussfirst = true;        //Hilfsvariable Nachgussbereitung während des Maischens
bool dclick;                      //Doppelclick gedrückt ?

long hwarten =  0;                //Wartezeitmessung Start

//Steigungsberechnung K/m -> Kelvin/Minute - Hinweis auf die Aufheizrate des Kessels

float alte_zeit;                  //Zeit bei t-1
float alte_zeit_nachguss;
float alte_temperatur;            //Temperatur bei t-1
float alte_nachgusstemperatur; 
unsigned int messzyklus = 1000; // Zeit im ms für dT und dt für die Gradientenberechnung - Vorgabe 1s


float gradient;                   // Steigungswert
float gradient_nachguss;
float kh = 1;                     //Gradientenfaktor kh (Heizen) - Vorgabe 1
float kk = 1;                     //Gradientenfaktor kk (Kühlen) - Vorgabe 1

//Logging -----------------------------------------------------------------------------

bool loggen = false;              //Daten Soll/Ist an Logprogramm übertragen zunächst deaktiviert

byte logprog = 0;                 //Auswahl des Logprogramms

bool serialcom = false;           //Logger SerialComInterface
bool logview = false;             //Logview Studio
bool udp_send;                    //Daten zu KBH2AiO

float logintervall = 2900;        //Logging Intervall 3s - 0,1 s in der Routine

//Doppelclick für temporären Aufruf des Setup-Menüs-------------------------------------
byte altmodus;                    //merkt sich den Modus für Doppelclick
String splitmodus;                //eingelesener Modus vom PC
byte zeitz = true;                //Zeitzählung nach Doppelclick nicht stoppen (zeitz = false)


//Notfallmodus nach Stromausfall ------------------------------------------------------
//Speichert den Zustand in Modus 28,29,30

byte Notfallmodus;                //Notfallmodus Maischeprogramm aktiv (es gab einen Stromausfall)
byte RastNr;                      //gespeicherte Rastnummer
byte RastTemp;                    //gespeicherte aktuelle Rasttemperatur
byte RastZeit;                    //gespeicherte aktuelle Rastzeit
byte RastZeit_h;                  //Rastzeit Stundenanteil
byte RastZeit_m;                  //Rastzeit Minutenanteil
byte RastZeit_s;                  //Rastzeit Sekundenanteil
int RastModus;                    //gespeicherter Modus
bool zeit_setzen;                 //Hilfsvariable für Settime

bool notfall_aktiv = false;       //soll der Notfallmodus genutzt werden? - nur im Setup ändern!

//WLAN --------------------------------------------------------------------------------

bool wlanAktiv = false;           //Vorgabe kein WLAN

char reply[500];                  //Daten zum PC
char antwort[500];                //Antwort vom PC
const bool printReply = true;
char command[20];
unsigned long lan_aufruf = 30000;  //Wartezeit WLAN-Steuerung

String responce;                   //enthält die Daten vom PC

bool uebertragungsfehler = false;//beim Übertragen von der AiO

String meldung_detail;            //Detailmeldungen in der AiO
String meldung;                   //Modusmeldungen in der AiO
String zeit_detail;               //Zeitablauf an AiO

//Kühlmodus ---------------------------------------------------------------------------

bool knot;                        //Kühlmodus nach Notfall gespeichert ?

//Jodprobe ----------------------------------------------------------------------------
int tz = 0;                       //Anzahl Piepser für Jodprobe - Vorgabe 3 in Modus 29
bool signalende;                  //nur einmal an Jodprobe erinnern

//Würzepumpe --------------------------------------------------------------------------
long int warten;
long int nachgusswarten;
long int pwarten;
bool anzeige;

bool manuell_ein = false;         //Manuelles Ein- und Ausschalten der Würzepumpe

//Malzrohrpumpe -----------------------------------------------------------------------

bool malzrohrpumpe_dauerbetrieb = false;  //Dauerbetrieb nur im Modus 0 ! - Reinigungslauf

bool ruf = false;
bool rufquittung = false;

//-------------------------------------------------------------------------------------
bool kuehlen = true;              //Auswahl für variable Temperaturregelung (Heizen oder Kühlen)

//-------------------------------------------------------------------------------------

//byte B_EIN;                 //Signalwert für den Buzzer (Piezo)


//setup================================================================================
void setup()
{
  Serial.begin(115200);


#ifdef dbg
  //-----------------Ausgabe Steckdosencode --------------------------------------------

  Serial.println("Steckdosencodierung ------------------------------------------------");
  Serial.println();
  Serial.println("             DEC       HEX      BIN");

  Serial.print("Heizung EIN: "); Serial.print(EEPROMReadlong(Heizung_EIN)); Serial.print(" / "); Serial.print(EEPROMReadlong(Heizung_EIN), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Heizung_EIN), BIN);
  Serial.print("Heizung AUS: "); Serial.print(EEPROMReadlong(Heizung_AUS)); Serial.print(" / "); Serial.print(EEPROMReadlong(Heizung_AUS), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Heizung_AUS), BIN);
  Serial.print("Mischer EIN: "); Serial.print(EEPROMReadlong(Umwaelz_EIN)); Serial.print(" / "); Serial.print(EEPROMReadlong(Umwaelz_EIN), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Umwaelz_EIN), BIN);
  Serial.print("Mischer AUS: "); Serial.print(EEPROMReadlong(Umwaelz_AUS)); Serial.print(" / "); Serial.print(EEPROMReadlong(Umwaelz_AUS), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Umwaelz_AUS), BIN);
  Serial.print("Ruf/Nachguss EIN: "); Serial.print(EEPROMReadlong(Nachguss_EIN)); Serial.print(" / "); Serial.print(EEPROMReadlong(Nachguss_EIN), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Nachguss_EIN), BIN);
  Serial.print("Ruf/Nachguss AUS: "); Serial.print(EEPROMReadlong(Nachguss_AUS)); Serial.print(" / "); Serial.print(EEPROMReadlong(Nachguss_AUS), HEX); Serial.print(" / "); Serial.println(EEPROMReadlong(Nachguss_AUS), BIN);
  Serial.println();
  Serial.println();
  Serial.println("Steckdosencodierung Ende--------------------------------------------");
  //------------------------------------------------------------------------------------
#endif

//  B_EIN = EEPROM.read(sound);
digitalWrite(RufR, B_EIN);


  if (MitDisplayEncoder)
  {
    // WLANSteuerung aktiv? ------------------------------------------------------------
    kbh2aio = EEPROM.read(Wlan);
    if (kbh2aio == 0) wlan_steuerung = false; else {
      wlan_steuerung = true;
      Serial2.begin(115200);
    }
  }
  else wlan_steuerung = true;  //wenn ohne Display und Encoder, dann WLAN immer EIN

  drehen = sollwert;

  encoder = new ClickEncoder(pinB, pinA, taster, stepsPerNotch);

#ifdef Tempsensor433   //wenn mit Funksender dann Timer3
  Timer3.initialize(1000);
  Timer3.attachInterrupt(timerIsr);
  Timer3.disablePwm(char(pinA));
  Timer3.disablePwm(char(pinB));
  funksensor = true;
#else
  Timer1.initialize(1000);   //sonst Timer1
  Timer1.attachInterrupt(timerIsr);
#endif

  last = -1;  //Hilfsvariable Drehgeber

  mySwitch.enableTransmit(sender);
  mySwitch.setRepeatTransmit(RESEND_SNIFFED_VALUES);   //Senden wiederholen


  //Relaisausgänge festlegen ---------------------------------------------------

  pinMode(HeizungR1, OUTPUT);
  pinMode(HeizungR2, OUTPUT);
  pinMode(NachgussR1, OUTPUT);
  pinMode(NachgussR2, OUTPUT);
  pinMode(UmwaelzR1, OUTPUT);
  pinMode(UmwaelzR2, OUTPUT);
  pinMode(fuellstand_sensor, INPUT_PULLUP);
  pinMode(fuellstand_pumpe, OUTPUT);
  pinMode(RufR, OUTPUT);
#ifdef geschwindigkeitsrelais
  pinMode(Speedrelais, OUTPUT);
#endif

  HeizungAUS();
  NachgussAUS();
  nachgussheizung = AUS;
  UmwaelzerAUS();

  mySwitch.send(funk_variabel_aus, 24);  //unabhängig vom Einsatzzweck (Funksteckdose als Ruf bzw. Nachguss) zunächst AUS
  WuerzepumpeEinAus(0);

  RufAUS();
  ruf = false;

#ifdef geschwindigkeitsrelais
  digitalWrite(Speedrelais, LOW_SPEED); //Normale Geschwindigkeit als Voreinstellung
  highspeed = false;
#endif

  //--------------------------------------------------------------------------------------

#ifdef Tempsensor433

  //Initialisierung des Temperaturempfängers -------------------------------------------

  // Initialisieren von IO und ISR
  vw_set_tx_pin(sender);
  vw_set_rx_pin(empfaengerPin);
  vw_setup(2000);  // Bits/sec

  vw_rx_start();       // Start des Empfaengers


#endif

  //------------------------------------------------------------------------------------
  //Stabilisierung des Drehencoders

  digitalWrite(pinA, HIGH);  //PullUp-Widerstand einschalten
  digitalWrite(pinB, HIGH);  //PullUp-Widerstand einschalten


  //LCD Setup --------------------------------------------------------------------------
  lcd.begin(20, 4);  //20 Spalten und 4 Zeilen

  //für Hinweis im Display +++++++++++++++++++++++++++
  if (H_AUS == LOW)
    hinweis = "Relais: AUS->LOW";
  else
    hinweis = "China : AUS->HIGH";


  lcd.setCursor(0, 0);
  lcd.print("AiO-MEGA  26.08.2017");
  lcd.setCursor(0, 1); lcd.print("Integr. G"); lcd.print(char(0xE1)); lcd.print("rf"); lcd.print(char(0xF5)); lcd.print("hrung");
  lcd.setCursor(0, 2); lcd.print(hinweis);  //Welche Relaislogik ist aktiv?

  lcd.setCursor(0, 3);
  lcd.print ("Allzeit Gut Sud");

  RufEIN();
  delay(2000);   //Rufdauer 2 sec.
  RufAUS();
  delay(2000);  //Startanzeige insges. 4 sec.

  //WLAN aktivieren --------------------------------------------------------------------

  if (wlan_steuerung)    //Einbinden des ESP8266 und konfiguration des UDP-Protokolls
  {
    wlanAktiv = true;
    connectWiFi();
  }

  lcd.clear();
  // -----------------------------------------------------------------------------------

  //Temperatursensor DS1820 ------------------------------------------------------------

  sensors.getAddress(insideThermometer, 0); //Thermometer Maischen, Kochen etc.
  sensors.getAddress(insideThermometer, 1); //Thermometer Nachguus (Heizlücke füllen!)
  sensors.setResolution(insideThermometer, 12);  //Auflösung 12 BIT (allerdings keine höhere Genauigkeit (+-0,5K)

  sensors.requestTemperatures(); // Temperatur holen

  sensorwert = sensors.getTempCByIndex(0);  //Überprüfung des Maischesensors
  if (int(sensorwert) != -127)  //wenn keine Störung vorliegt
  {
    sensorwert = sensors.getTempCByIndex(1);  //Nachgusssensor vorhanden?
    if (sensorwert != -127)  anzahlTempsensoren = 2; else anzahlTempsensoren = 1;    //wenn Nachgusssensor vorhanden, dann Anzahl Sensoren = 2
  } else anzahlTempsensoren = 0;  //wenn Fehler Maischesensor, dann kein Sensor aktiv


  //------------------------------------------------------------------------------------

  //Rührparameter einlesen -------------------------------------------------------------

  if (EEPROM.read(udauer) == 0) dauerbetrieb = 'N'; else dauerbetrieb = 'J';

  intervalEin = EEPROM.read(umwaelzer_ie);
  intervalAus = EEPROM.read(umwaelzer_ia);

  if (intervalEin == 0) (intervalEin = 60);
  if (intervalAus == 0) (intervalAus = 60);

  // Kochschwelle einlesen --------------------------------------------------------------
  kschwelle = EEPROM.read(Kochschwelle);

  if (kschwelle == 0) (kschwelle = 98);  //Vorgabe wenn noch nichts im EEPROM

  //Funksteckdosencode einlesen --------------------------------------------------------

  init_funk();

  //Wiedereinschaltverzögerung Heizungsrelais -------------------------------------------

  if (EEPROMReadlong(wein_h) > 100000) EEPROMWritelong(wein_h, 10000);
  esvheizen = EEPROMReadlong(wein_h);

  //Wiedereinschaltverzögerung Kühlung --------------------------------------------------

  if (EEPROMReadlong(kuehlverz) > 300000) EEPROMWritelong(kuehlverz, 300000);
  kzeitverz = EEPROMReadlong(kuehlverz);

  if (EEPROMReadlong(graf_h) > 100) EEPROMWritelong(graf_h, kh * 10); //Vorgabewert Gradientenfaktor für Heizen
  kh = EEPROMReadlong(graf_h) / 10;

  if (EEPROMReadlong(graf_k) > 100) EEPROMWritelong(graf_k, kk * 10); //Vorgabewert Gradientenfaktor für Heizen
  kk = EEPROMReadlong(graf_k) / 10;

  //Logging verwenden ? -----------------------------------------------------------------

  loggen = EEPROM.read(logging);
  if (loggen)
  {
    logprog = EEPROM.read(loggerwahl);

    switch (logprog)
    {
      case 0 : {
          serialcom = true;  //SerialComInterface
          logview = false;
          break;
        }
      case 1 : {
          logview = true;    //Logview
          serialcom = false;
          break;
        }
    }

  }

  //Pumpe oder Rührwerk ? ------------------------------------------------------------------

  wahl = EEPROM.read(umwaelzwahl);
  if (wahl == 0) {
    ruehrer = true;
    pumpe = false;
  } else {
    ruehrer = false;
    pumpe = true;
  }

  //----------------------------------------------------------------------------------------

  //Funkruf oder Nachgussbereitung ---------------------------------------------------------

  wahl = EEPROM.read(dritte_funksteckdose);
  if (wahl == 1) {
    funkruf = true;
  } else {
    funkruf = false;
  }
  //----------------------------------------------------------------------------------------

  //Funksteckdosen Heizung/Rührer/Variabel Aus----------------------------------------------

  HeizungAUS();
  NachgussAUS();
  nachgussheizung = AUS;

  rwerk = 1;
  umwaelzerEinAus(0);                     //zunächst natürlich AUS
  rwerk = 0;
  //-------------------------------------------------------------------------------------


  if (EEPROM.read(k_aktiv) == 1) {     //Wiedereinschalten Kühlung/Heizung nach Stromausfall?
    modus = KUEHLEN;
    knot = false;
    drehen = EEPROM.read(g_kuehltemp);
  } else knot = true; //Stromausfall im Kühlmodus ?

  //Rührer/Pumpe während des Einmaischens aktiv? -----------------------------------------

  ruehrer_beim_einmaischen = EEPROM.read(ruepumeinmaisch);

  //---------------------------------------------------------------------------------------

  //Einsprungstelle nach Stromausfall Maischen/Gären------------------------------------

  notfall_aktiv = EEPROM.read(notfall);   //soll nach einem Stromausfall der letzte Programmpunkt aufgerufen werden?
  //Die Werte werden jede Minute gespeichert.
  if (notfall_aktiv)
  {
    Notfallmodus = EEPROM.read(notfallmodus);
    if (Notfallmodus == 1)
    {
      RastNr = EEPROM.read(rastnr);
      RastTemp = EEPROM.read(rasttemp_aktuell);
      RastZeit = EEPROM.read(rastzeit_aktuell);
      RastZeit_h = EEPROM.read(zeit_h);
      RastZeit_m = EEPROM.read(zeit_m);
      RastZeit_s = EEPROM.read(zeit_s);
      maischtemp = EEPROM.read(MaischTemp);
      rastZeit[1] = EEPROM.read(rastzeit_1);
      rastTemp[1] = EEPROM.read(rasttemp_1);
      rastZeit[2] = EEPROM.read(rastzeit_2);
      rastTemp[2] = EEPROM.read(rasttemp_2);
      rastZeit[3] = EEPROM.read(rastzeit_3);
      rastTemp[3] = EEPROM.read(rasttemp_3);
      rastZeit[4] = EEPROM.read(rastzeit_4);
      rastTemp[4] = EEPROM.read(rasttemp_4);
      rastZeit[5] = EEPROM.read(rastzeit_5);
      rastTemp[5] = EEPROM.read(rasttemp_5);
      rastZeit[6] = EEPROM.read(rastzeit_6);
      rastTemp[6] = EEPROM.read(rasttemp_6);
      rastTemp[RastNr] = RastTemp;
      rastZeit[RastNr] = RastZeit;
      rasten = EEPROM.read(Rasten);
      RastModus = EEPROMReadlong(rastmodus);
      hopfengabe[0] = EEPROM.read(hopfengabe1);
      hopfengabe[1] = EEPROM.read(hopfengabe2);
      hopfengabe[2] = EEPROM.read(hopfengabe3);
      hopfengabe[3] = EEPROM.read(hopfengabe4);
      hopfengabe[4] = EEPROM.read(hopfengabe5);
      hopfengabe[5] = EEPROM.read(hopfengabe6);
      kochzeit = EEPROM.read(Kochzeit);

      modus = RastModus;
      anfang = 0;
      x = RastNr;
      rastTemp[x] = RastTemp;
      rastZeit[x] = RastZeit;

      if (modus > ANSTELLTEMPERATUR)
      {
        gaerRastTemp[x] = RastTemp;
        gaerRastZeit[x] = RastZeit;
      }
      zeit_setzen = true;
    } else zeit_setzen = false;
  }
  //für Gradientenberechnung ------------------------------------------------------------

  alte_zeit = millis(); //beim ersten Anlauf etwas warten
  alte_zeit_nachguss = millis();

  alte_temperatur = 25;  // Dämpft die erste Gradientenberechnung
  alte_nachgusstemperatur = 25;
  //-------------------------------------------------------------------------------------

  x = 1;  //Startwert Rasten

  kuehlen = EEPROM.read(heiz_kuehl);  //Kühl- oder Heizfunktion
}

//Ende Setup=============================================================================

//loop===================================================================================
void loop()
{

#ifdef Tempsensor433  //nur für Funksensor

  //Empfangspuffer ---------------------------
  uint8_t buf[VW_MAX_MESSAGE_LEN];
  uint8_t buflen = VW_MAX_MESSAGE_LEN;

  if (vw_get_message(buf, &buflen)) // Non-blocking
  {
    int i;
    String wert = "";

    for (i = 0; i < buflen; i++) wert += char(buf[i]);
    sensorwert = wert.toFloat();
  }

#endif

  drehen += encoder->getValue();

  if (drehen != last) {
    last = drehen;
  }

  ButtonPressed = false;
  ClickEncoder::Button b = encoder->getButton();
  if (b != ClickEncoder::Open)
  {
    switch (b)
    {
      case ClickEncoder::Pressed:
        break;

      case ClickEncoder::Held:
        altmodus = modus;
        modus = ABBRUCH;
        break;

      case ClickEncoder::Released:
        ButtonPressed = false;
        break;

      case ClickEncoder::Clicked:
        ButtonPressed = true;

#ifdef geschwindigkeitsrelais
        if (ruehrer)  //wenn als Mischer "Rührer" gewählt wurde, kann die Geschwindigkeit mit einem Click umgeschaltet werden.
        {
          if ((modus == MAISCHEN_START) || (modus == EINMAISCHEN) || (modus == TEMPERATURAUTOMATIK) || (modus == ZEITAUTOMATIK) || (modus == ENDTEMPERATURAUTOMATIK))
          {
            if (highspeed) {
              digitalWrite(Speedrelais, LOW_SPEED);
              meldung_detail = "Rührer-Low-Speed";
            } else {
              digitalWrite(Speedrelais, HIGH_SPEED);
              meldung_detail = "Rührer-High-Speed";
            }
            highspeed = !highspeed;
          }
        }
#else
        if (((modus == MAISCHEN_START) || (modus == EINMAISCHEN) || (modus == TEMPERATURAUTOMATIK) || (modus == ZEITAUTOMATIK) || (modus == ENDTEMPERATURAUTOMATIK)) && (heizung == AUS))
        {
          if (button == true) {
            button = false;
            dauerbetrieb = 'J';
          } else {
            button = true;
            dauerbetrieb = 'N';
          }
        }
        else button = false;
#endif

        if (modus == WUERZEPUMPE) //in diesem Modus kann durch einen Click zwischen Automatik und Manuell umgeschaltet werden
            if (!manuell_ein) manuell_ein = true; else manuell_ein = false;
        break;

      case ClickEncoder::DoubleClicked:
        if (modus != SETUPMENUE)  //gilt nicht für das Setupmenü
        {
          if (modus != KUEHLEN)  //Doppelclick führt zum Setupmenü (nicht im Modus "Kühlen/Heizen")
          {
            altmodus = modus; //Merker für letzten Modus
            tempdrehen = drehen; //Merker für aktuellen Wert von drehen
            anfang = 0;
            dclick = true;  //Doppelclick wurde betätigt
            modus =  SETUPMENUE;
            zeitz = false;
          } else
          { //im Modus "Kühlen/Heizen" wird zwischen "Heizen" und "Kühlen" umgeschaltet
            lcd.setCursor(0, 2); lcd.print("Modus = ");
            kuehlen = !kuehlen;
            if (kuehlen) {
              lcd.print("K");
              lcd.print(char(0xF5));
              lcd.print("hlen");
              EEPROM.write(heiz_kuehl, 1);
            } else {
              lcd.print("Heizen");
              EEPROM.write(heiz_kuehl, 0);
            }
          }
          break;
        }
    }
  }

  // Temperatursensor DS1820 Temperaturabfrage ---------------------------------------
  sensors.requestTemperatures(); // Temperatur holen

  if (anzahlTempsensoren > 0)
  {
#ifndef Tempsensor433 //ohne Funksensor
    if (anzahlTempsensoren == 1) sensorwert = sensors.getTempCByIndex(0);  //kein Funksensor - Temperatursensor Kabel holt Maischetemperatur
#endif
#ifdef Tempsensor433  //mit Funksensor
    if (anzahlTempsensoren == 1) nachgusssensor = sensors.getTempCByIndex(0); //Funksensor und nur ein Kabelsensor - Kabelsensor holt Nachgusstemperatur
#endif


    if (anzahlTempsensoren == 2) {       // Zwei Kabelsensoren und kein Funksensor
      sensorwert = sensors.getTempCByIndex(0);
      nachgusssensor = sensors.getTempCByIndex(1);
    }
  }


#ifdef Tempsensor433  //mit Funksensor
  Serial.println("Funksensor aktiviert!");
#endif

  if ((sensorwert != isttemp) && (n < 3))  // Mittelwert aus 3 Messungen des Maischesensorwertes - Messfehlervermeidung
    // für den Nachgusssensor nicht erforderlich
  {
    n++;                                   // nach mehreren
  }                                        // Messungen
  else
  {
    isttemp = sensorwert;
    n = 0;
  }
  //----------------------------------------------------------------------------------

  // Zeitermittlung ------------------------------------------------------------------
  sekunden = second();    //aktuell Sekunde abspeichern für die Zeitrechnung
  minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
  stunden = hour();       //aktuell Stunde abspeichern für die Zeitrechnung
  //----------------------------------------------------------------------------------

  if (modus == NACHISOMERISIERUNG)  //Anzeige der Temperatur während der Nachisomerisierung
  {
    lcd.setCursor(9, 3);
    lcd.print("ist ");
    lcd.print(float(sensorwert), 1);
    if (sensorwert > 10)
    {

      lcd.print(char(0xDF));
      lcd.print("C");
    }
    else
    {
      lcd.print(char(0xDF));
      lcd.print("C ");
    }
  }


  //Kochfunktion - ohne Regelung -------------------------------------------------------
  if ((modus > KOCHZEIT) && (modus < NACHISOMERISIERUNG))
  {
    HeizungEIN();
    lcd.setCursor(0, 3); lcd.print("Heizen");

    if ((alte_zeit + messzyklus) < millis()) {
    gradient = (isttemp - alte_temperatur) / (millis() - alte_zeit) * 60000;  //Gradient/min. dT/dt
      alte_zeit = millis();
      alte_temperatur = isttemp;
    }
    if (modus > STARTKOCHEN)  //Gradientenanzeige auch während des Kochens
    {
      lcd.setCursor(0, 1);
      lcd.print(gradient);
      if (modus < KOCHEN) lcd.print(" K/m           "); else  lcd.print(" K/m   ");
    }
    lcd.setCursor(9, 3);
    lcd.print("ist ");
    lcd.print(float(sensorwert), 1);
    if (sensorwert > 10)
    {

      lcd.print(char(0xDF));
      lcd.print("C");
    }
    else
    {
      lcd.print(char(0xDF));
      lcd.print("C ");
    }
  }

  //---------------------------------------------------------------------------------

  //Regelung Gärführung -------------------------------------------------------------

  if (modus >= ANSTELLTEMPERATUR)
  { // Temperaturanzeige Istwert für Gärführung

    lcd.setCursor(9, 3);
    lcd.print("ist ");
    lcd.print(float(sensorwert), 1);
    if (sensorwert > 10)
    {

      lcd.print(char(0xDF));
      lcd.print("C");
    }
    else
    {
      lcd.print(char(0xDF));
      lcd.print("C ");
    }

    if ((isttemp >= (sollwert - gaerhysterese)) && (isttemp <= (sollwert + gaerhysterese))) {  //in diesem Fenster sind
      if (gaerregelung == EIN)                                                                 //Heizung und Kühlung AUS
      {
        gaerregelung = AUS;
        lcd.setCursor(0, 3); lcd.print("        ");
        HeizungAUS();
        heizung = AUS;
        UmwaelzerAUS();
        lcd.setCursor(0, 3); lcd.print("        ");
        rwerk = 0;
      }
    } else {
      gaerregelung = EIN;
    }

    if ((gaerregelung == EIN) && (isttemp > sollwert))   //Isttemperatur liegt über Solltemperatur
    {
      if (heizung == EIN)
      {
        if (heizung == EIN) {
          lcd.setCursor(0, 3);
          lcd.print("        ");
        }
        HeizungAUS();
        heizung = AUS;
        kwarten = millis();
      }
      if (millis() > (kwarten + kzeitverz))
      {
        kwarten = millis();
        UmwaelzerEIN(); //Umwaelzrelais nun als Kühlrelais konfiguriert
        lcd.setCursor(0, 3); lcd.print("K"); lcd.print(char(0xF5)); lcd.print("hlen");
        rwerk = 1;
      }
    }


    if ((gaerregelung == EIN) && (isttemp < sollwert))   //Isttemperatur liegt unter Solltemperatur
    {
      UmwaelzerAUS();
      rwerk = 0;

      if (heizung == AUS)
      {
        if (heizung == AUS) {
          lcd.setCursor(0, 3);
          lcd.print("        ");
        }
        if (millis() > (hwarten + esvheizen))
        {
          hwarten = millis();
          HeizungEIN();
          lcd.setCursor(0, 3); lcd.print("Heizen");
          heizung = EIN;
        }

      }
    }
  }

  if ((modus == HAUPTMENUE) || (modus == BRAUMENUE) || (modus == OPTIONENMENUE))
  {
    lcd.setCursor(9, 3);
    lcd.print("ist ");
    lcd.print(float(sensorwert), 1);
    if (sensorwert > 10)
    {

      lcd.print(char(0xDF));
      lcd.print("C");
    }
    else
    {
      lcd.print(char(0xDF));
      lcd.print("C ");
    }
  }

  if (regelung == EIN)  //Alle Modi mit Regelung - ausser Gärführung
  {
    if (loggen)  //wenn loggen dann im Intervall 5 s
    {
      aktuelleMillis = millis();

      if (aktuelleMillis - alteMillis >= logintervall) {
        alteMillis = aktuelleMillis;
        if (serialcom) logger_serial();
        if (logview) logger_logview();
      }
    }

    if (heizung == EIN)  //zur Sicherheit überprüfen wir den aktuellen Zustand der Funksteckdosen
    {
      if ((heizungInterval1 + 5000) < millis())
      {
        heizungInterval1 = millis();
        mySwitch.send(funk_heizung_ein, 24);
        delay(100);
        if (ruehrer)
        {
          rwerk = 0;
          umwaelzerEinAus(1);
        }
        if (nachgussheizung == AUS) NachgussAUS();
      }
    }

    if (heizung == AUS)
    {
      if ((heizungInterval2 + 5000) < millis())
      {
        heizungInterval2 = millis();
        mySwitch.send(funk_heizung_aus, 24);
        delay(100);
        if (nachgussheizung == EIN) NachgussEIN();
      }
    }



    // Sensorfehler --------------------------------------------------------------------
    // Sensorfehler -127 => VCC fehlt
    // Sensorfehler 85.00 => interner Sensorfehler ggf. Leitung zu lang
    //                       nicht aktiviert
    // Sensorfehler 0.00 => Datenleitung oder GND fehlt

    if (regelung == EIN) //nur bei Modi mit Temperaturregelung
    {
      if ((int)isttemp == -127 || (int)isttemp == 0 ) {
        //zur besseren Erkennung Umwandling in (int)-Wert
        //sonst Probleme mit der Erkennung gerade bei 0.00
        if (sensorfehler == 0)
        {
          rufmodus = modus;
          lcd.clear();
          lcd.setCursor(7, 0);
#ifndef Tempsensor433   //Mit Funksensor abgeschaltet
          lcd.print("Sensorfehler!");
          meldung_detail = "Sensorfehler-Maischesensor";
#else
          lcd.setCursor(0, 0); lcd.print("Funksensor Fehler!");
          meldung_detail = "Funkensorfehler-Maischesensor";
#endif
          regelung = AUS;
          heizung = AUS;
          rwerk = 1;
          umwaelzerEinAus(0);
          sensorfehler = 1;
          rufmodus = ABBRUCH;
          modus = RUFALARM;
        }
      } else sensorfehler = 0;
    }

    //-------------------------------------------------------------------------------

    if (sensorfehler == 0)
    {
      // Temperaturanzeige Istwert ----------------------------------------------------

      lcd.setCursor(8, 3);
      //      lcd.print("ist ");
      if (rwerk == 1)
      {
        if (dauerbetrieb == 'J') lcd.print("MD");
        else lcd.print("MI");
      }
      else lcd.print("  ");
      lcd.setCursor(13, 3); lcd.print(float(sensorwert), 1);
      if (sensorwert > 10)
      {

        lcd.print(char(0xDF));
        lcd.print("C");
      }
      else
      {
        lcd.print(char(0xDF));
        lcd.print("C ");
      }
    }
    //--------------------------------------------------------------------------------
    if ((modus >= MAISCHEN_START) && (modus <= ENDTEMPERATURAUTOMATIK))  //Maischeautomatik
      if (heizung == AUS)
        if (anzahlTempsensoren == 2)  //Nachgussbereitung in den Ausschaltlücken der Maischeheizung
          //wenn zwei Temperatursensoren verbaut sind (inkl. Funksensor)
        {
          if ((alte_zeit_nachguss + messzyklus) < millis()) {
            gradient = (isttemp - alte_temperatur) / (millis() - alte_zeit) * 60000;  //Gradient/min. dT/dt
            alte_zeit_nachguss = millis();
            alte_nachgusstemperatur = nachgusssensor;
          }

          if (nachgusssensor >= (endtemp - (gradient_nachguss))) {
            if (nachgussheizung == EIN) {
              nachgussheizung = AUS;
              nachgusswarten = millis();
              nachguss_sollwerterreicht = true;
            }

          }
          if (nachgusssensor <= (endtemp - (gradient_nachguss))) {
            if ((millis() > (nachgusswarten + esvheizen)) || nachgussfirst) {
              nachgussfirst = false;
              if (nachgusssensor < endtemp) nachgussheizung = EIN;
            }
            nachguss_sollwerterreicht = false;
          }
        }



    //Heizregelung----------------------------------------------------------------------
    if (regelung == EIN)
    {
      // Temperaturanzeige Sollwert ----------------------------------------------------
      lcd.setCursor(15, 1);
      lcd.print(float(sollwert), 0);
      lcd.print(char(0xDF)); lcd.print("C ");

      //--------------------------------------------------------------------------------

      if ((modus == EINMAISCHTEMPERATUR) || (modus == EINMAISCHEN) || (modus == TEMPERATURAUTOMATIK) || (modus == ZEITAUTOMATIK) || (modus == ENDTEMPERATURAUTOMATIK) || (modus == NACHGUSS) || (modus == KUEHLEN))  //Maischeautomatik/Nachguss/Kühlen
      {
        lcd.setCursor(0, 0);
        if ((modus != NACHGUSS ) && (modus != KUEHLEN)) lcd.print("Maischen");
        if ((alte_zeit + messzyklus) < millis()) {
          gradient = (isttemp - alte_temperatur) / (millis() - alte_zeit) * 60000;  //Gradient/min. dT/dt
          alte_zeit = millis();
          alte_temperatur = isttemp;
        }
        if (modus != NACHGUSS) {
          lcd.setCursor(0, 1);
          lcd.print(gradient);
          lcd.print(" K/m   ");
        }
      }

      //Gradientenregelung--------------------------------------------------------------

      if (modus != KUEHLEN) { //Heizungsregelung
        wechsel = false;
        if (isttemp >= (sollwert - (gradient * kh))) {
          if (heizung == EIN) {
            heizung = AUS;
            if (nachgussheizung == AUS)
            {
              nachgussheizung = EIN;
              wechsel = true;
            }
            hwarten = millis();
            if (isttemp >= sollwert) sollwerterreicht = true;
          }

        }

        if (isttemp < (sollwert - (gradient * kh))) {
          if ((millis() > (hwarten + esvheizen)) || first) {
            first = false;
            if (isttemp <= sollwert) {
              heizung = EIN;
              if (nachgussheizung == EIN)
              {
                nachgussheizung = AUS;
                wechsel = true;
              }
            }
          }
          if (isttemp < sollwert) sollwerterreicht = false;
          if ((sondermodus == EINMAISCHEN) && (modus == RUFALARM)) {
            heizung = AUS;
            nachgussheizung = EIN;
          }
        }
      }  //Modus != Kühlen

      if (modus == KUEHLEN) {  //Heiz-/Kühlregelung
        if (kuehlen)
        {
          if (isttemp < (sollwert - (gradient * kk))) {
            if (heizung == EIN) {
              heizung = AUS;
              kwarten = millis();
            }
          }

          if (isttemp >= (sollwert - (gradient * kk))) {
            if ((millis() > (kwarten + kzeitverz)) || first) {
              kwarten = millis();
              first = false;
              if (isttemp >= sollwert) heizung = EIN;
            }
          }
        } else
        {
          if (isttemp > (sollwert - (gradient * kh))) {
            if (heizung == EIN) {
              heizung = AUS;
              hwarten = millis();
              sollwerterreicht = true;
            }

          }
          if (isttemp <= (sollwert - (gradient * kh))) {
            if ((millis() > (hwarten + esvheizen)) || first) {
              first = false;
              heizung = EIN;
            }
            sollwerterreicht = false;
          }
        }

      }
    }


    // Zeigt "Heizen" wenn Heizen und "Kühlen wenn Kühlen ausserhalb der Gärführung
    if (heizung == EIN)
    {
      lcd.setCursor(0, 3);
      if (modus != KUEHLEN) {
        lcd.print("Heizen  ");
      } else  {
        if (kuehlen)
        {
          lcd.print("K");
          lcd.print(char(0xF5));
          lcd.print("hlen");
        } else lcd.print("Heizen  ");
      }

      if ((!eingeschaltet) && (modus != ABBRUCH)) //bei Modus KUEHLEN verlassen keine Anzeige und kein Einschalten der Heizung
      {
        NachgussAUS();
        if (wechsel) delay(esv_nachguss);
        HeizungEIN();
        eingeschaltet = true;
      }

      if ((modus != KUEHLEN) && (ruehrer)) if (rwerk == 0) umwaelzerEinAus(1);  //nicht im Kühlmodus
      if ((modus != KUEHLEN) && (pumpe) && (dauerbetrieb == 'N')) intervallbetrieb();
      if ((modus != KUEHLEN) && (pumpe) && (dauerbetrieb == 'J')) umwaelzerEinAus(1);
    }
    else
    {
      lcd.setCursor(0, 3);
      if (anzahlTempsensoren == 1) lcd.print("       ");
      if ((anzahlTempsensoren == 2) || (anzahlTempsensoren == 1 && funksensor))
      {
        lcd.setCursor(0, 3);
        lcd.print("N:");
        lcd.print(float(nachgusssensor), 1); lcd.print(char(0xDF)); lcd.print("C");
      }

      if (eingeschaltet)
      {
        HeizungAUS();
        if (wechsel) delay(esv_nachguss);

        if ((modus >= MAISCHEN_START) && (modus <= ENDTEMPERATURAUTOMATIK)) NachgussEIN();
        eingeschaltet = false;
      }

      if (modus != KUEHLEN)
      { //nicht im Kühlmodus
        if (ruehrer)
        {
          if (dauerbetrieb == 'J') {
            rwerk = 0; // alter Status des Rührers
            // damit nicht fortwährend Daten gesendet werden!
            umwaelzerEinAus(1)
            ;
          } //Rührer im Dauerbetrieb
          else if ((heizung == AUS) && (regelung == EIN) && (modus != NACHGUSS) && (sollwerterreicht) && (dauerbetrieb == 'N')) intervallbetrieb(); //Rührer im Intervallbetrieb
        }
      } //Ende Umwaelz
      if ((pumpe) && (regelung == EIN) && (dauerbetrieb == 'N')) intervallbetrieb();
      if ((ruehrer) && (regelung == EIN) && (dauerbetrieb == 'N')) intervallbetrieb();
      if ((pumpe) && (regelung == EIN) && (dauerbetrieb == 'J'))
      {
        rwerk = 0; // alter Status des Rührers
        umwaelzerEinAus(1);
      }
    }


    //Ende Heizregelung--------------------------------------------------------------

  } //Regelung = EIN

  // Abfrage Modus ------------------------------------------------------------------

  encoder->setAccelerationEnabled(true);
  switch (modus) {
    case HAUPTMENUE: {
        meldung = "Hauptmenü";
        encoder->setAccelerationEnabled(false);  //Geschwindigkeitsabhängigkeit beim Drehen des Encoders AUS
        regelung = AUS;  //Hauptmenü
        funktion_hauptschirm();
        break;
      }

    case BRAUMENUE: {
        meldung = "Braumenü";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Braumenü
        funktion_braumenue();
        break;
      }

    case OPTIONENMENUE: {
        meldung = "Optionentmenü";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Optionenmenü
        funktion_optionsmenue();
        break;
      }



    case KUEHLEN: {
        meldung = "Kühlen/Heizen";
        encoder->setAccelerationEnabled(true);   //Geschwindigkeitsabhängigkeit beim Drehen des Encoders EIN
        regelung = EIN;  //Kühlen
        funktion_temperatur();
        break;
      }

    case NACHGUSS: {
        meldung = "Nachguss";
        encoder->setAccelerationEnabled(true);
        regelung = EIN;  //Nachguss
        funktion_temperatur();
        break;
      }

    case WUERZEPUMPE: {
        meldung = "Würzepumpe";
        regelung = AUS;  //Würzepumpe
        wuerzepumpe();
        break;
      }

    case MPUMPE: { //Malzrohrpumpe Reinigungslauf
        meldung = "Malzrohrpumpe_Dauerbetrieb";
        regelung = AUS;
        malzrohrDauerbetrieb();
        break;
      }

    case SETUPMENUE: {
        meldung = "Setupmenü";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Einstellungen
        funktion_SetupMenu();
        break;
      }

    case EXTERNMENUE: {
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Einstellungen
        funktion_Externmenu();
        break;
      }

    case GRADIENTENFAKTOR_HEIZEN: {
        meldung = "Gradientenfaktor-Heizen";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Gradientenfaktor Heizen
        khwert();
        break;
      }

    case EINSCHALTVERZOEGERUNG_HEIZRELAIS: {
        meldung = "Einschaltverzögerung-Heizrelais";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Wartezeit Wiedereinschalten Heizungsrelais
        ESVHeiz();
        break;
      }

    case GRADIENTENFAKTOR_KUEHLEN: {
        meldung = "Gradientenfaktor-Kühlen";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Gradientenfaktor Kühlen
        kkwert();
        break;
      }

    case WARTEZEIT_KUEHLEN: {
        meldung = "Wartezeit-Kühlen";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Wartezeit bein Kühlen vor dem Wiedereinschalten
        wartenKuehlen();
        break;
      }

    case RASTANZAHL: {
        meldung = "Rastzahl-eingeben";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Anzahl Rasten
        funktion_rastanzahl();
        break;
      }
    case EINMAISCHTEMPERATUR: {
        meldung = "Einmaischtemperatur";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe Einmaischtemperatur
        funktion_maischtemperatur();
        break;
      }
    case RASTTEMPERATUREN: {
        meldung = "Rasttemperaturen";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe der Temperatur der Rasten
        funktion_rasteingabe();
        break;
      }

    case RASTZEITEN: {
        meldung = "Rastzeiten";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe der Rastzeitwerte
        funktion_zeiteingabe();
        break;
      }

    case AUSWAHL_UMWAELZER: {
        meldung = "Rührwerk-oder-Pumpe";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Rührwerk oder Malzrohrpumpe?
        auswahl_umwaelzer();
        break;
      }

    case UMWAELZER_DAUERLAUF: {
        meldung = "Mischer-Dauerlauf?";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Rührwerk Dauerlauf?
        umwaelzer();
        break;
      }

    case UMWAELZER_INTERVALL_EIN: {
        meldung = "Rührwerl-Intervall-Ein";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Rührwerk Intervall EIN
        umwaelzer_ein();
        break;
      }

    case UMWAELZER_INTERVALL_AUS: {
        meldung = "Rührwerk-Intervall-Aus";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Rührwerk Intervall AUS
        umwaelzer_aus();
        break;
      }

    case RUEPUMAKTIV: {
        meldung = "Rührwerk-Intervall-Aus";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Rührwerk Intervall AUS
        RuePumAktiv();
        break;
      }

    case DRITTE_FUNKSTECKDOSE: {
        meldung = "Auswahl-Steckdose";
        //Auswahl Verwendung der 3. Funksteckdose
        steckdosenwahl();
        break;
      }

    case FRAGESTECKDOSENCODEEINGEBEN: {
        //soll der Steckdosencode nun eingegeben werden?
        frageCodeEingeben();
        break;
      }

    case STECKDOSENCODE: {
        meldung = "Steckdosencode-eingeben";
        //Steckdosencode eingeben
        steckdosencode();
        break;
      }

    case KOCHSCHWELLE: {
        meldung = "Kochschwelle-eingeben";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Kochschwelle - ab hier Brauerruf
        kochschwelle();
        break;
      }

    case WECHSEL_TEMP_ZEIT: {
        meldung = "Braumeisterruf";
        regelung = AUS;  //Eingabe Braumeisterruf An/Aus
        wechsel_temp_zeit_eingabe();
        break;
      }

    case CONNECT_WIFI: {
        // WiFi starten
        connectWiFi();
        break;
      }

    case WIFI_EIN_AUS: {
        //WiFi Ein bzw. Aus
        wifieinaus();
        break;
      }

    case ABMAISCHTEMPERATUR: {
        meldung = "Abmaischtemperatur";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Abmaischtemperatur
        funktion_endtempeingabe();
        break;
      }

    case MAISCHEN_START: {
        meldung = "Start-Maischen";
        regelung = AUS;  //Start Maischeprozess
        funktion_startabfrage();
        break;
      }

    case EINMAISCHEN: {
        meldung = "Maischetemperatur-Automatik";
        encoder->setAccelerationEnabled(true);
        regelung = EIN;  //Automatischer Ablauf Maischen
        funktion_einmaischen();
        break;
      }

    case TEMPERATURAUTOMATIK: {
        meldung = "Temperaturautomatik";
        encoder->setAccelerationEnabled(true);
        regelung = EIN;
        funktion_tempautomatik();
        break;
      }

    case ZEITAUTOMATIK: {
        meldung = "Zeitautomatik";
        encoder->setAccelerationEnabled(true);
        regelung = EIN;
        funktion_zeitautomatik();
        break;
      }

    case ENDTEMPERATURAUTOMATIK: {
        meldung = "Endtemperatur-Automatik";
        encoder->setAccelerationEnabled(true);
        regelung = EIN;
        funktion_endtempautomatik();
        break;
      }

    case RUFALARM: {
        meldung = "Rufalarm";
        regelung = AUS;
        funktion_braumeisterrufalarm();
        break;
      }

    case BRAUMEISTERRUFALARM: {
        meldung = "Rufalarm";
        regelung = AUS;
        funktion_braumeisterruf();
        break;
      }

    case ANZAHL_HOPFENGABEN: {
        meldung = "Anzahl-Hopfengaben";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;
        anzahl_hopfengaben();
        break;
      }

    case  NACHISOZEIT: {
        meldung = "Nachisozeit";
        meldung_detail = "Nachisomerisierung";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;
        nachisozeit();
        break;
      }

    case HOPFENGABE: {
        meldung = "Hopfengabe!";
        regelung = AUS;
        funktion_hopfengabe();
        break;
      }

    case KOCHZEIT: {
        meldung = "Kochzeit";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;
        funktion_kochzeit();
        break;
      }

    case STARTKOCHEN: {
        meldung = "Start-Kochen";
        regelung = AUS;
        funktion_startkochen();
        break;
      }

    case KOCHEN: {
        meldung = "Kochen";
        meldung_detail = "Kochen-gestartet";
        regelung = AUS;
        funktion_kochen();
        break;
      }

    case NACHISOMERISIERUNG: {
        meldung = "Nachisomerisierung";
        meldung_detail = "Nachisomerisierung";
        regelung = AUS;
        nachiso();
        break;
      }

    case LOGGING_EIN_AUS: {
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //Logger aktiviert?
        LoggingEinAus();
        break;
      }

    case LOGWAHL: {
        encoder->setAccelerationEnabled(false);
        regelung = AUS;  //LogViewEinAus
        Logwahl();
        break;
      }

    case  NOTFALLMODUS: {
        meldung = "Notfallmodus";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;
        StromAus();  //Notfallmodus Ein/Aus
        break;
      }

    case WLANSTEUERUNG_AKTIV: {
        encoder->setAccelerationEnabled(false);
        regelung = AUS;
        WLANSteuerungAktiv();  //WLAN-Steuerung für KBH2AiO aktiv?
        break;
      }

    case ABBRUCH: {
        meldung = "Abbruch";
        encoder->setAccelerationEnabled(false);
        regelung = AUS;
        funktion_abbruch();
        break;
      }

    case GAERFUEHRUNG: {
        meldung = "Gärführung_START";
        regelung = AUS;
        funktion_gaerrastanzahl();
        break;
      }

    case ANLAUFGAERSTUFEN: {
        regelung = AUS;
        meldung_detail = "Gaerstufe_anlaufen";
        AnlaufGaerstufen();
        break;
      }

    case GAERSTARTTEMPERATUR: {
        meldung = "Anstelltemperatur";
        meldung_detail = "Anstelltemperatur_erreicht";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe Gärstarttemperatur
        gaerstarttemperatur();
        break;
      }

    case GAERZEITEN: {
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe der Gärzeitwerte
        gaerzeiteingabe();
        break;
      }

    case GAERTEMPERATUREN: {
        meldung = "Gaertemperaturen";
        encoder->setAccelerationEnabled(true);
        regelung = AUS;  //Eingabe der Gärzeitwerte
        gaerrasteingabe();
        break;
      }

    case WECHSEL_GAER_TEMP_ZEIT: {
        meldung = "Gaerzeitautomatik";
        regelung = AUS;
        wechsel_gaer_temp_zeit();
        break;
      }

    case ANSTELLTEMPERATUR: {
        regelung = AUS;
        meldung = "Anstelltemperatur";
        anlauf_anstelltemperatur();
        break;
      }

    case GAERZEITAUTOMATIK: {
        encoder->setAccelerationEnabled(true);
        regelung = AUS;
        meldung = "Gaerzeitautomatik";
        gaerzeitautomatik();
        break;
      }

    case SOUNDEINSTELLUNG: {
        encoder->setAccelerationEnabled(true);
        regelung = AUS;
        sound_einstellen();
        break;
      }
  }


  //Datenaustausch AiO -> KBH2AiO ---------------------------------------------------

  if (!udp_send)
    if (wlanAktiv)
      if (wlan_steuerung)
      {
        if ((lan_aufruf + PCZyklus) < millis())  //Daten zum PC senden
        {
          lan_aufruf = millis();
          zum_pc();  //von der AiO zum PC
        }
        zurAiO(AIOZyklus);  //Rezept aus KBH und Menübefehle vom PC
      }
  //---------------------------------------------------------------------------------
}
// Ende Loop
//--------------------------------------------------------------------------------


//Funktionen=======================================================================
// ab hier Funktionen

void funktion_hauptschirm()      //Modus=HAUPTMENUE
{
  if ((anfang == 0) || (dclick))

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Brauen");
    lcd.setCursor(2, 1);
    lcd.print("Optionen");
    lcd.setCursor(2, 2);
    lcd.print("Setup");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
    dclick = false;
  }


  if (drehen == 255)
  {
    drehen = 2;
  }
  if (drehen == 3)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = BRAUMENUE;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = OPTIONENMENUE;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }
  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = SETUPMENUE;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    modus = rufmodus;
    anfang = 0;
    lcd.clear();
  }
}
//-----------------------------------------------------------------------------------

// Funktion Braumenü------------------------------------------------------------
void funktion_braumenue()      //Modus=BRAUMENUE
{

  if ((anfang == 0) || (dclick))

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Maischen");
    lcd.setCursor(2, 1);
    lcd.print("Nachguss");
    lcd.setCursor(2, 2);
    lcd.print("Kochen");
    lcd.setCursor(13, 0);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
    dclick = false;
  }


  if (drehen == 255)
  {
    drehen = 3;
  }
  if (drehen == 4)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(11, 0);
    lcd.print("  ");
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = RASTANZAHL;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = NACHGUSS;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = KOCHZEIT;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = HAUPTMENUE;
    lcd.setCursor(11, 0);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    modus = rufmodus;
    if (modus == NACHGUSS)
    { //Übergabe an Modus NACHGUSS
      drehen = nachgusssollwert;                    //Nachgusstemperatur Sollwert
    }
    anfang = 0;
    lcd.clear();
  }
}
//-----------------------------------------------------------------------------------

// Funktion Optionsmenü------------------------------------------------------------
void funktion_optionsmenue()      //Modus=OPTIONENMENUE
{

  if ((anfang == 0) || (dclick))

  {
    lcd.setCursor(2, 0);
    lcd.print("Var.K/H");
    lcd.setCursor(2, 1);
    lcd.print("MPumpe");  //Malzrohrpumpe Dauerbetrieb EIN-AUS
    lcd.setCursor(2, 2);
    lcd.print("SPumpe");  //Sudpumpe!
    lcd.setCursor(13, 0);
    lcd.print("G"); lcd.print(char(0xE1)); lcd.print("rf"); lcd.print(char(0xF5)); lcd.print("hr"); //Gärführung

    lcd.setCursor(13, 1);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
    dclick = false;
  }


  if (drehen == 255)
  {
    drehen = 4;
  }
  if (drehen == 5)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(11, 0);
    lcd.print("  ");
    lcd.setCursor(11, 1);
    lcd.print("  ");
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = KUEHLEN;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 4;
    rufmodus = MPUMPE;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = WUERZEPUMPE;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = GAERFUEHRUNG;
    lcd.setCursor(11, 0);
    lcd.print("=>");
  }

  if (drehen == 4)
  {
    altdrehen = 4;
    rufmodus = HAUPTMENUE;
    lcd.setCursor(11, 1);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    modus = rufmodus;
    if (modus == KUEHLEN)
    { //Übergabe an Modus kuehlen/heizen
      drehen = EEPROM.read( g_kuehltemp); //ganzzahliger Vorgabewert für Kühlen
      EEPROM.write(k_aktiv, 1); //Kühlmodus bei Stromausfall aktiv
      altmodus = KUEHLEN;
    }

    anfang = 0;
    lcd.clear();
  }
}
//-----------------------------------------------------------------------------------

//Funktion SetupMenu-----------------------------------------------------------------

void funktion_SetupMenu()      //Modus = SETUPMENUE
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Schwelle");
    lcd.setCursor(2, 1);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("h/Pump");
    lcd.setCursor(2, 2);
    lcd.print("Regler");
    lcd.setCursor(2, 3);
    lcd.print("Logging");
    lcd.setCursor(13, 0);
    lcd.print("Extern");
    lcd.setCursor(13, 1);
    lcd.print("Notfall");
    lcd.setCursor(13, 2);
    lcd.print("Sound");
    lcd.setCursor(13, 3);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
  }


  if (drehen == 255)
  {
    drehen = 7;
  }
  if (drehen == 8)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(0, 3);
    lcd.print("  ");
    lcd.setCursor(11, 0);
    lcd.print("  ");
    lcd.setCursor(11, 1);
    lcd.print("  ");
    lcd.setCursor(11, 2);
    lcd.print("  ");
    lcd.setCursor(11, 3);
    lcd.print("  ");
    altdrehen = drehen;
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = KOCHSCHWELLE;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = AUSWAHL_UMWAELZER;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = GRADIENTENFAKTOR_HEIZEN;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = LOGGING_EIN_AUS;
    lcd.setCursor(0, 3);
    lcd.print("=>");
  }

  if (drehen == 4)
  {
    altdrehen = 4;
    rufmodus = EXTERNMENUE;
    lcd.setCursor(11, 0);
    lcd.print("=>");
  }

  if (drehen == 5)
  {
    altdrehen = 5;
    rufmodus = NOTFALLMODUS;
    lcd.setCursor(11, 1);
    lcd.print("=>");
  }

  if (drehen == 6)
  {
    altdrehen = 6;
    rufmodus = SOUNDEINSTELLUNG;
    lcd.setCursor(11, 2);
    lcd.print("=>");
  }

  if (drehen == 7)
  {
    altdrehen = 7;
    rufmodus = altmodus;
    lcd.setCursor(11, 3);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    if (dclick) {
      anfang = 1;
    } else {
      anfang = 0;
    }
    modus = rufmodus;
    drehen = tempdrehen;
    first = true;
    lcd.clear();
  }

}

//--------------------------------------------------------------------------------

//Funktion Externmenü-------------------------------------------------------------

void funktion_Externmenu()      //Modus = EXTERNMENUE
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("WLAN-Steuerung");
    lcd.setCursor(2, 1);
    lcd.print("Funk");
    lcd.setCursor(2, 2);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
  }

  if (drehen == 255) drehen = 2;
  if (drehen == 3) drehen = 0;

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");

    altdrehen = drehen;
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = WLANSTEUERUNG_AKTIV;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = DRITTE_FUNKSTECKDOSE;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }


  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = SETUPMENUE;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    anfang = 0;
    modus = rufmodus;
    lcd.clear();
  }

}

//--------------------------------------------------------------------------------

// Funktion nur Temperaturregelung------------------------------------------------
void funktion_temperatur()      //Modus= KUEHLEN und NACHGUSS (separater Nachguss!)
{
  if (anfang == 0)
  {
    if (modus == NACHGUSS) drehen = nachgusssollwert;
    anfang = 1;
  }
  if (drehen == 255) drehen = 0;
  sollwert = drehen;


  if (modus == KUEHLEN)
  {
    lcd.setCursor(0, 0);
    if (kuehlen) {
      lcd.print("K");
      lcd.print(char(0xF5));
      lcd.print("hlen");
    } else lcd.print("Heizen");
    altmodus = KUEHLEN; // Merker für Doppelclick
    lcd.setCursor(0, 2); lcd.print("Modus = ");
    if (kuehlen) {
      lcd.print("K");
      lcd.print(char(0xF5));
      lcd.print("hlen");

    } else {
      lcd.print("Heizen");

    }
  }

  if (modus == NACHGUSS)
  {
    lcd.setCursor(0, 0);
    lcd.print("Nachguss");

  }

  if ((modus == NACHGUSS) && (isttemp >= sollwert) && (nachgussruf == false))   //Nachgussruf -> Sollwert erreicht
  {
    meldung_detail = "Temperatur-erreicht!";
    nachgussruf = true;
    rufmodus = modus;          //Rufalarm
    altmodus = modus;
    modus = RUFALARM;
    y = 0;
    braumeister[y] = 2;        //nur Ruf und weiter mit Regelung
  }

}
//---------------------------------------------------------------------------------

void malzrohrDauerbetrieb()
{
  if (anfang == 0)
  {
    if (pumpe) {
      lcd.clear();
      lcd.setCursor(0, 0); lcd.print("Malzrohrpumpe");
      lcd.setCursor(0, 1); lcd.print("Reinigungslauf");
      lcd.setCursor(0, 3); lcd.print("Ende Taste halten");
      UmwaelzerEIN();
    } else
    {
      lcd.setCursor(0, 0); lcd.print("keine Pumpe gew.!");
      lcd.setCursor(0, 3); lcd.print("Ende Taste halten");
    }
  }
  anfang = 1;
}

//---------------------------------------------------------------------------------
void wuerzepumpe()  //Modus = WUERZEPUMPE
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Pumpe: ");
    anfang = 1;
    anzeige = true;
    warten = millis();
    pwarten = millis();
    drehen = esvpumpen / 1000;
  }

  if (manuell_ein)
  {
    WuerzepumpeEinAus(1);
    lcd.setCursor(0, 1); lcd.print("Dauerbetrieb EIN");
    lcd.setCursor(7, 0);
    lcd.print("EIN");
    lcd.setCursor(0, 2); lcd.print("                  ");
    meldung_detail = "Pumpe_Dauerbetrieb";
  }
  else

  {
    lcd.setCursor(0, 1); lcd.print("Automatik       ");
    digitalWrite(fuellstand_sensor, HIGH);  //PullUp-Widerstand wenn Signal vom Sensor = GND (0V)
    //wenn das Signal vom Sensor = 5V dann realen PullDown-Widerstand (10K) vom Anschluss gegen GND (0V)
    int val = digitalRead(fuellstand_sensor);
    meldung_detail = "Pumpe_Automatik";

    lcd.setCursor(0, 2); lcd.print("ESV: "); lcd.setCursor(6, 2); lcd.print(" s");
    if (drehen == 255) drehen = 0;
    lcd.setCursor(5, 2); lcd.print(drehen);

    esvpumpen = drehen * 1000;

    //ist die Pumpe bei Sensor LOW aktiv, dann
    if (val == Sensor_EIN) {
      lcd.setCursor(0, 1);
      if (millis() > warten + 1000) {
        if (anzeige)
        {
          lcd.print("Bitte warten! ");
        }
        warten = millis();
      } else {
        if (anzeige)
        {
          lcd.print("               ");
        }
      }

      if (millis() > (pwarten + esvpumpen))
      {
        anzeige = false;
        WuerzepumpeEinAus(1);
        lcd.setCursor(0, 1); lcd.print("                 ");
        lcd.setCursor(7, 0);
        lcd.print("EIN");
        meldung_detail = "Pumpe-EIN";
      }
    }
    if (val == Sensor_AUS) {
      WuerzepumpeEinAus(0);
      lcd.setCursor(7, 0);
      meldung_detail = "Pumpe-Autom.";
      lcd.print("AUS");
      pwarten = millis();
      anzeige = true;
    }
  }
}

//--------------------------------------------------------------------------------

// Funktion Eingabe der Rastanzahl------------------------------------------------
void funktion_rastanzahl()          //Modus = RASTANZAHL
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(12, 1);
    lcd.print("Rasten");

    drehen = rasten;
    anfang = 1;
    dclick = false;
  }

  //Vorgabewerte bei verschiedenen Rasten

  if (rasten != drehen)
  {
    if ((int)drehen == 1)
    {
      rastTemp[1] = 67;
      rastZeit[1] = 60;
      maischtemp = 65;
    }
    if (drehen == 2)
    {
      rastTemp[1] = 62;
      rastZeit[1] = 30;
      rastTemp[2] = 72;
      rastZeit[2] = 35;
      maischtemp = 62;
    }
    if (drehen == 3)
    {
      rastTemp[1] = 55;
      rastZeit[1] = 10;
      rastTemp[2] = 62;
      rastZeit[2] = 35;
      rastTemp[3] = 72;
      rastZeit[3] = 25;
      maischtemp = 55;
    }
    if (drehen == 4)
    {
      rastTemp[1] = 40;
      rastZeit[1] = 20;
      rastTemp[2] = 55;
      rastZeit[2] = 15;
      rastTemp[3] = 64;
      rastZeit[3] = 35;
      rastTemp[4] = 72;
      rastZeit[4] = 25;
      maischtemp = 35;
    }
    if (drehen == 5)
    {
      rastTemp[1] = 35;
      rastZeit[1] = 20;
      rastTemp[2] = 40;
      rastZeit[2] = 20;
      rastTemp[3] = 55;
      rastZeit[3] = 15;
      rastTemp[4] = 64;
      rastZeit[4] = 35;
      rastTemp[5] = 72;
      rastZeit[5] = 25;
      maischtemp = 30;
    }

    if (drehen == 6)
    {
      rastTemp[1] = 35;
      rastZeit[1] = 20;
      rastTemp[2] = 40;
      rastZeit[2] = 20;
      rastTemp[3] = 55;
      rastZeit[3] = 15;
      rastTemp[4] = 64;
      rastZeit[4] = 35;
      rastTemp[5] = 72;
      rastZeit[5] = 25;
      rastTemp[6] = 78;
      rastZeit[6] = 10;
      maischtemp = 30;
    }
  }

  rasten = drehen;


  if (rasten <= 1)
  {
    rasten = 1;
    drehen = 1;
  }
  if (rasten >= 6)
  {
    rasten = 6;
    drehen = 6;
  }

  lcd.setCursor(19, 1);
  lcd.print(rasten);

  if (ButtonPressed)
  {
    modus = EINMAISCHTEMPERATUR;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Maischtemperatur--------------------------------------------------------
void funktion_maischtemperatur()      //Modus = EINMAISCHTEMPERATUR
{

  if ((anfang == 0) || (dclick))
  {
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(0, 1);
    lcd.print("Einmaischen         ");
    drehen = maischtemp;
    first = true;
    anfang = 1;
    dclick = false;
  }

  maischtemp = drehen;

  if (maischtemp <= 30)
    maischtemp = 30;

  if (maischtemp >= 78)
    maischtemp = 78;

  lcd.setCursor(16, 1);
  lcd.print(maischtemp);
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus = RASTTEMPERATUREN;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Rasteingabe Temperatur--------------------------------------------------
void funktion_rasteingabe()      //Modus = RASTTEMPERATUREN
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = rastTemp[x];
    anfang = 1;
    dclick = false;
  }

  rastTemp[x] = drehen;

  if (rastTemp[x] < 30)
  {
    rastTemp[x] = 30;
    drehen = 30;
  }
  if (rastTemp[x] > 78)
  {
    rastTemp[x] = 78;
    drehen = 78;
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");
  lcd.setCursor(14, 1);
  lcd.print(int(rastTemp[x]));
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus = RASTZEITEN;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Rasteingabe Zeit--------------------------------------------------------
void funktion_zeiteingabe()      //Modus = RASTZEITEN
{

  if ((anfang == 0) || (dclick))
  {
    drehen = rastZeit[x];
    anfang = 1;

    lcd.setCursor(0, 0);
    lcd.print("Eingabe");

    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");
    lcd.setCursor(14, 1);
    lcd.print(int(rastTemp[x]));
    lcd.print(char(0xDF)); lcd.print("C ");


    dclick = false;
  }

  rastZeit[x] = drehen;

  if (rastZeit[x] <= 1)
  {
    rastZeit[x] = 1;
    drehen = 1;
  }
  if (rastZeit[x] >= 99)
  {
    rastZeit[x] = 99;
    drehen = 99;
  }

  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  if (ButtonPressed)
  {
    modus = WECHSEL_TEMP_ZEIT;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
void umwaelzer() //Modus = UMWAELZERWAHL
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Umwaelzer");
    lcd.setCursor(0, 1);
    lcd.print("Dauerbetrieb");
    drehen = EEPROM.read(udauer);
    anfang = 1;
    dclick = false;
  }

  char antwort = ja_nein[drehen % 2];

  lcd.setCursor(14, 1); lcd.print(antwort);

  dauerbetrieb = antwort;

  if (ButtonPressed)
  {
    if (dauerbetrieb == 'N') {
      modus = UMWAELZER_INTERVALL_EIN;
      if (EEPROM.read(udauer) != 0) EEPROM.write(udauer, 0);
    } else {

      if (!pumpe) modus = RUEPUMAKTIV; else modus = SETUPMENUE; //Pumpe während des Einmaischens immer deaktiviert!
      if (EEPROM.read(udauer) != 1) EEPROM.write(udauer, 1);
    }
    speichern(2);
    anfang = 0;
  }
}

//--------------------------------------------------------------------------------

// Funktion Rührwerk Intervall EIN------------------------------------------------
void umwaelzer_ein()      //Modus = UMWAELZER_INTERVALL_EIN
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    if (ruehrer) {
      lcd.print("R");
      lcd.print(char(0xF5));
      lcd.print("hrwerk");
    }
    if (pumpe) {
      lcd.print("Pumpe");
    }
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalEin;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;
  intervalEin = drehen;

  if (intervalEin > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s   ");
  }

  if (ButtonPressed)
  {
    if (EEPROM.read(umwaelzer_ie) != intervalEin) EEPROM.write(umwaelzer_ie, intervalEin);
    speichern(2);
    modus = UMWAELZER_INTERVALL_AUS;
    anfang = 0;
  }
}

//--------------------------------------------------------------------------------

// Funktion Rührwerk Intervall AUS------------------------------------------------

void umwaelzer_aus()      //Modus = UMWAELZER_INTERVALL_AUS
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    if (ruehrer) {
      lcd.print("R");
      lcd.print(char(0xF5));
      lcd.print("hrwerk");
    }
    if (pumpe) {
      lcd.print("Pumpe");
    }
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalAus;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;

  intervalAus = drehen;

  if (intervalAus > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s   ");
  }

  if (ButtonPressed)
  {
    if (EEPROM.read(umwaelzer_ia) != intervalAus) EEPROM.write(umwaelzer_ia, intervalAus);
    speichern(2);
    if (!pumpe) modus = RUEPUMAKTIV; else {
      modus = SETUPMENUE;
      if (EEPROM.read(ruepumeinmaisch) != 0) EEPROM.write(ruepumeinmaisch, 0);
    }
    anfang = 0;
  }
}

//--------------------------------------------------------------------------------


//Funktion Steckdosencode eingeben -----------------------------------------------

void steckdosencode() //Modus = STECKDOSENCODE
{
  if (anfang == 0)
  {
    lcd.clear();
    mySwitch.enableReceive(empfaenger_int);
    anfang = 1;
    drehen = 1;
  }
  else
  {

    if (drehen == 1) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Heizung EIN");
      meldung_detail = "Heizung-EIN-drücken";
      adr = Heizung_EIN;

    }
    if (drehen == 2) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Heizung AUS");
      meldung_detail = "Heizung-AUS-drücken";
      adr = Heizung_AUS;
    }
    if (drehen == 3) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Umwaelz EIN");
      meldung_detail = "Umwaelzer-EIN-drücken";
      adr = Umwaelz_EIN;
    }
    if (drehen == 4) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Umwaelz AUS");
      meldung_detail = "Umwaelzer-AUS-drücken";
      adr = Umwaelz_AUS;
    }

    if (!funkruf)
    {
      if (drehen == 5) {
        lcd.setCursor(0, 1);
        lcd.print("Taste Nachguss EIN");
        meldung_detail = "Nachguss-EIN-drücken";
        adr = Nachguss_EIN;
      }
      if (drehen == 6) {
        lcd.setCursor(0, 1);
        lcd.print("Taste Nachguss AUS");
        meldung_detail = "Nachguss-AUS-drücken";
        adr = Nachguss_AUS;
      }
    }
    else
    {
      if (drehen == 5) {
        lcd.setCursor(0, 1);
        lcd.print("Taste Ruf EIN");
        meldung_detail = "Ruf-EIN-drücken";
        adr = Nachguss_EIN;
      }
      if (drehen == 6) {
        lcd.setCursor(0, 1);
        lcd.print("Taste Ruf AUS");
        meldung_detail = "Ruf-AUS-drücken";
        adr = Nachguss_AUS;
      }
    }

    if (drehen > 6) drehen = 1;

    value = mySwitch.getReceivedValue();
    if (value) {
      // value in EPROM schreiben
      if (EEPROMReadlong(adr) != value) EEPROMWritelong(adr, value);
      lcd.setCursor(0, 2);
      lcd.print("speichern");
      meldung_detail = "Code-gespeichert";

      signalton(500);
      drehen++;
      if (drehen == 7) {
        modus = HAUPTMENUE;
        anfang = 0;
        mySwitch.disableReceive();  //Empfänger wieder deaktivieren
      }
      lcd.clear();
    }
    mySwitch.resetAvailable();
    init_funk();

  }
}
//--------------------------------------------------------------------------------

//Funktion Kochschwelle einstellen -----------------------------------------------

void kochschwelle()  //Modus = KOCHSCHWELLE
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Kochschwelle");
    lcd.setCursor(0, 1); lcd.print("Ruf bei:");
    lcd.setCursor(12, 1); lcd.print(char(0xDF)); lcd.print("C ");
    drehen = EEPROM.read(Kochschwelle); //EPROM-Speicher auslesen
    anfang = 1;
    dclick = false;
  }
  else
  {
    if (drehen > 99) drehen = 20;
    if (drehen < 20) drehen = 99;

    kschwelle = drehen;
    lcd.setCursor(9, 1); lcd.print(kschwelle);

    if (ButtonPressed)
    {
      if (EEPROM.read(Kochschwelle) != kschwelle) EEPROM.write(Kochschwelle, kschwelle);
      speichern(2);
      modus = SETUPMENUE;
      anfang = 0;
    }
  }
}
//--------------------------------------------------------------------------------

// Funktion Intervallbetrieb der Umwälzeinheit------------------------------------

void intervallbetrieb()  //kein separator Modus
{
  if (regelung == EIN)
  {
    unsigned long ein = 1000 * (unsigned long)intervalEin;
    unsigned long aus = 1000 * (unsigned long)intervalAus;

    if (firstinterval)
    {
      intervalstart = millis(); firstinterval = false;
    }
    else if ((intervalstart  + ein)  >= millis()) {
      if (rwerk == 0) umwaelzerEinAus(1);
    }
    else if ((intervalstart  + ein + aus) >= millis()) {
      if (rwerk == 1) umwaelzerEinAus(0);
    }
    else firstinterval = true;
  }
}

//--------------------------------------------------------------------------------

void umwaelzerEinAus(int rueh)  // kein separator Modus
{
  if ((modus == RUFALARM) && (altmodus != EINMAISCHEN)) // bei Ruf Nachgußbereitung den Rührer nicht einschalten
  {
    UmwaelzerAUS();

  }
  else if ((regelung == EIN) && (modus != NACHGUSS) && (modus != KUEHLEN) && (modus != ABBRUCH)) // außer bei Nachgußbereitung/kühlen
  {
    if (rueh == 1)
    {
      if (rwerk == 0)
      {
        // Rührer wird eingeschaltet
        UmwaelzerEIN();
        rwerk = 1;
      }


    }
    if (rueh == 0)
    {
      if (rwerk == 1)
      {
        //Rührer wird ausgeschaltet
        UmwaelzerAUS();
        rwerk = 0;
      }
    }
  }
}

//--------------------------------------------------------------------------------

// Funktion Braumeister-----------------------------------------------------------

void wechsel_temp_zeit_eingabe() //Modus = WECHSEL_TEMP_ZEIT
{

  braumeister[x] = 0;
  if (x < rasten)
  {
    x++;
    modus = RASTTEMPERATUREN;           //Sprung zur Rasttemperatureingabe
    anfang = 0;
  }
  else
  {
    x = 1;
    modus = ABMAISCHTEMPERATUR;                //Sprung zur Rastzeiteingabe
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Ende Temperatur-------------------------------------------------------
void funktion_endtempeingabe()      //Modus = ABMAISCHTEMPERATUR
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = endtemp;
    anfang = 1;
  }

  endtemp = drehen;

  if (endtemp <= 10)
    endtemp = 10;

  if (endtemp >= 80)
    endtemp = 80;

  lcd.setCursor(0, 1);
  lcd.print("Abmaischen");
  lcd.setCursor(16, 1);
  lcd.print(int(endtemp));
  lcd.print(char(0xDF)); lcd.print("C ");


  if (ButtonPressed)
  {
    modus = MAISCHEN_START;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Startabfrage----------------------------------------------------------
void funktion_startabfrage()      //Modus = MAISCHEN_START
{
  if (anfang == 0)
  {
    lcd.clear();
    anfang = 1;
    nachgussheizung = EIN;
    lcd.setCursor(0, 0);
    lcd.print("Aufheizen zum");
    lcd.setCursor(0, 1);
    lcd.print("Einmaischen");
  }
  aktuelleMillis = millis();

  if (aktuelleMillis - alteMillis >= blinkintervall) {
    alteMillis = aktuelleMillis;
    lcd.setCursor(0, 3);
    if (textsichtbar) {
      lcd.print("Taste = Start");
      textsichtbar = false;
    } else  {
      lcd.print("             ");
      textsichtbar = true;
    }
  }

  if (ButtonPressed)
  {
    if (notfall_aktiv)
    {
      {
        Notfallmodus = EEPROM.read(0);
        if (EEPROM.read(notfallmodus) != 1) EEPROM.write(notfallmodus, 1); //Notfallmodus aktiviert
        EEPROM.write(Rasten, rasten);
        EEPROM.write(MaischTemp, maischtemp);

        switch (rasten) {
          case 1: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
            }

          case 2: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
              EEPROM.write(rastzeit_2, rastZeit[2]);
              EEPROM.write(rasttemp_2, rastTemp[2]);
            }

          case 3: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
              EEPROM.write(rastzeit_2, rastZeit[2]);
              EEPROM.write(rasttemp_2, rastTemp[2]);
              EEPROM.write(rastzeit_3, rastZeit[3]);
              EEPROM.write(rasttemp_3, rastTemp[3]);
            }

          case 4: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
              EEPROM.write(rastzeit_2, rastZeit[2]);
              EEPROM.write(rasttemp_2, rastTemp[2]);
              EEPROM.write(rastzeit_3, rastZeit[3]);
              EEPROM.write(rasttemp_3, rastTemp[3]);
              EEPROM.write(rastzeit_4, rastZeit[4]);
              EEPROM.write(rasttemp_4, rastTemp[4]);
            }


          case 5: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
              EEPROM.write(rastzeit_2, rastZeit[2]);
              EEPROM.write(rasttemp_2, rastTemp[2]);
              EEPROM.write(rastzeit_3, rastZeit[3]);
              EEPROM.write(rasttemp_3, rastTemp[3]);
              EEPROM.write(rastzeit_4, rastZeit[4]);
              EEPROM.write(rasttemp_4, rastTemp[4]);
              EEPROM.write(rastzeit_5, rastZeit[5]);
              EEPROM.write(rasttemp_5, rastTemp[5]);
            }

          case 6: {
              EEPROM.write(rastzeit_1, rastZeit[1]);
              EEPROM.write(rasttemp_1, rastTemp[1]);
              EEPROM.write(rastzeit_2, rastZeit[2]);
              EEPROM.write(rasttemp_2, rastTemp[2]);
              EEPROM.write(rastzeit_3, rastZeit[3]);
              EEPROM.write(rasttemp_3, rastTemp[3]);
              EEPROM.write(rastzeit_4, rastZeit[4]);
              EEPROM.write(rasttemp_4, rastTemp[4]);
              EEPROM.write(rastzeit_5, rastZeit[5]);
              EEPROM.write(rasttemp_5, rastTemp[5]);
              EEPROM.write(rastzeit_6, rastZeit[6]);
              EEPROM.write(rasttemp_6, rastTemp[6]);
            }
        }
      }
    }

    anfang = 0;
    modus = EINMAISCHEN;
  }
}
//--------------------------------------------------------------------------------


// Funktion Automatik Maischtemperatur--------------------------------------------
void funktion_einmaischen()      //Modus = EINMAISCHEN
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    meldung_detail = "Aufheizen_Einmaischen";
    drehen = maischtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
    if (notfall_aktiv) EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
  }
  if (drehen > 80) drehen = 30;
  maischtemp = drehen;

  sollwert = maischtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    altmodus = modus;
    sondermodus = modus;   //Sonderbehandlung im Modus "Rufalarm"!
    modus = TEMPERATURAUTOMATIK;
    rufmodus = modus;
    y = 0;
    braumeister[y] = 2;
    modus = RUFALARM;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Funktion Automatik Temperatur--------------------------------------------------
void funktion_tempautomatik()      //Modus = TEMPERATURAUTOMATIK
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");
    meldung_detail = String(x) + "_Rast_anfahren";
    firstinterval = true;  //Intervallbetrieb Rührwerk
    rwerk = 0;
    drehen = rastTemp[x];
    anfang = 1;
    first = true;
    dclick = false;
    if (notfall_aktiv)
    {
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
      EEPROM.write(rastnr, x);
      EEPROM.write(rasttemp_aktuell, rastTemp[x]);
      EEPROM.write(rastzeit_aktuell, rastZeit[x]);
    }
  }
  if (drehen > 78) drehen = 30;
  rastTemp[x] = drehen;

  sollwert = rastTemp[x];


  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    modus = ZEITAUTOMATIK;                //zur Zeitautomatik
    anfang = 0;
    signalton(1000);
  }
}
//--------------------------------------------------------------------------------

// Funktion Automatik Zeit--------------------------------------------------------
void funktion_zeitautomatik()      //Modus = ZEITAUTOMATIK
{
  if (anfang == 0)
  {
    drehen = rastZeit[x];              //Zuordnung für Encoder
    signalende = false;
    if (altx != x) {
      meldung_detail = String(x) + "_Rast_erreicht";  //bei Rastwechsel Zeit wieder bei 0 starten!
      zeitz = true;
    }

    if (notfall_aktiv)
    {
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
      EEPROM.write(rastnr, x);
      EEPROM.write(rasttemp_aktuell, rastTemp[x]);
      EEPROM.write(rastzeit_aktuell, rastZeit[x]);
    }

    // Initialisiere Zeitzählung------------------------------------------------------------------

    if (zeitz) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    if (zeit_setzen) {
      RastZeit_h = EEPROM.read(zeit_h);
      RastZeit_m = EEPROM.read(zeit_m);
      RastZeit_s = EEPROM.read(zeit_s);
      setTime(RastZeit_h, RastZeit_m, RastZeit_s, 00, 01, 01);
      zeit_setzen = false;
    }

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    lcd.setCursor(0, 2);
    lcd.print("        ");
    anfang = 1;
  }

  altx = x;
  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");


  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  lcd.setCursor(0, 2);

  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }


  if (stunden == 0)
    minuten = minutenwert;
  else
    minuten = ((stunden * 60) + minutenwert);

  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(1, 2);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(1, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(0, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }
  String m;
  String s;
  m = String(minutenwert);
  s = String(sekunden);
  if (minutenwert < 10) m = "0" + String(minutenwert);   //Aktuelle Zeit zum PC
  if (sekunden < 10) s = "0" + String(sekunden);
  zeit_detail = m + ":" + s;

  // Ende Zeitzählung---------------------------------------------------------------

  //Zustand für den Fall von Stromausfall speichern---------------------------------

  if (notfall_aktiv)
  {
    if (sekunden == 0)  //einmal pro Minute speichern
    {
      EEPROM.write(rastnr, x);
      EEPROM.write(rasttemp_aktuell, rastTemp[x]);
      EEPROM.write(rastzeit_aktuell, rastZeit[x]);
      EEPROM.write(zeit_h, stunden);
      EEPROM.write(zeit_m, minuten);
      EEPROM.write(zeit_s, sekunden);
    }
  }

  //--------------------------------------------------------------------------------

  //Erinnerung Jodprobe-------------------------------------------------------------

  if (x == rasten)
  {
    if ((minuten >= rastZeit[x] - 5) && (!signalende)) //5 min. vor Ende der letzten Rast
    {
      lcd.setCursor(0, 3); lcd.print("Jodprobe!");
      meldung_detail = "Jodprobe!";
      for (tz = 0; tz <= 3; tz++) {
        signalton(500);
        delay(200);
      }
      signalende = true;
      lcd.setCursor(0, 3); lcd.print("         ");
    }


  }

  //--------------------------------------------------------------------------------


  if (drehen > 100) drehen = 0;
  rastZeit[x] = drehen;   //Encoderzuordnung


  if (minuten >= rastZeit[x])  // Sollwert erreicht ?
  {
    anfang = 0;
    y = x;
    signalton(1000);



    if (x < rasten)
    {
      modus = TEMPERATURAUTOMATIK;                // zur Temperaturregelung
      x++;                    // nächste Stufe
    }
    else
    {
      x = 1;                              //Endtemperatur
      modus = ENDTEMPERATURAUTOMATIK;     //Endtemperatur
    }

    if (braumeister[y] > 0)
    {
      rufmodus = modus;
      modus = RUFALARM;
    }
  }
}
//--------------------------------------------------------------------------------

// Funktion Automatik Endtemperatur-----------------------------------------------
void funktion_endtempautomatik()      //Modus = ENDTEMPERATURAUTOMATIK
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(10, 0);
    lcd.print("Abmaischen");

    drehen = endtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
    if (notfall_aktiv)
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
  }
  if (drehen > 78) drehen = 78;
  endtemp = drehen;

  sollwert = endtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    meldung_detail = "Abmaischen_Abbruch=ENDE";
    altmodus = modus;
    modus = ABBRUCH;                //Abbruch
    rufmodus = modus;
    modus = RUFALARM;
    regelung = AUS;              //Regelung aus
    heizung = AUS;               //Heizung aus
    y = 0;
    braumeister[y] = 1;
  }
}
//--------------------------------------------------------------------------------

// Funktion braumeisterrufalarm---------------------------------------------------

void funktion_braumeisterrufalarm()      //Modus = RUFALARM
{
  if (anfang == 0)
  {
    rufsignalzeit = millis();
    altsekunden = millis();
    anfang = 1;
  }

  if (sondermodus == EINMAISCHEN) //Einmaischtemperatur erreicht ! Heizung AUS Rührer EIN oder AUS bis "weiter"
  {
    altmodus = RUFALARM;
    heizung = AUS;
    if (ruehrer_beim_einmaischen)
    {
      rwerk = 1;
      UmwaelzerEIN();
    } else
    {
      rwerk = 0;
      UmwaelzerAUS();
    }
  }


  if (millis() >= (altsekunden + 1500)) //Blinken der Anzeige und RUF
  {
    lcd.setCursor(17, 2);
    lcd.print("   ");
    RufAUS();
    ruf = false;
    if (millis() >= (altsekunden + 1500))
    {
      altsekunden = millis();
      pause++;
    }
  }
  else
  {
    lcd.setCursor(17, 2);
    lcd.print("Ruf");
    if (pause <= 4)
    {
      RufEIN();
      ruf = true;
    }
    if (sondermodus != EINMAISCHEN)
      if (pause > 8)
        pause = 0;
  }                                   //Blinken der Anzeige und RUF

  //20 Sekunden Rufsignalisierung wenn "Ruf Signal"
  if (braumeister[y] == 2 && millis() >= (rufsignalzeit + 20000))
  {
    if (sondermodus != EINMAISCHEN)
    {
      anfang = 0;
      pause = 0;
    }
    RufAUS();
    ruf = false;
    lcd.setCursor(17, 2);
    lcd.print("   ");
  }
  //weiter mit Programmablauf

  if ((ButtonPressed) || (rufquittung))
  {
    pause = 0;
    anfang = 0;
    RufAUS();
    ruf = false;
    if (braumeister[y] == 2)
    {
      lcd.setCursor(17, 2);
      lcd.print("   ");
      if (sensorfehler == 1) {
        sensorfehler = 0;  //Sensorfehler oder Alarmtest
        lcd.setCursor(8, 0);
        lcd.print("            ");
      }
      modus = rufmodus;
      if (modus == NACHGUSS) anfang = 1;
    }
    else
    {
      modus = BRAUMEISTERRUFALARM;

    }
  }
  if (rufquittung) {
    modus = rufmodus;
    anfang = 3;
    if (sondermodus == EINMAISCHEN) {
      anfang = 0;
      sondermodus = 0;
      modus = TEMPERATURAUTOMATIK;
    }
    if (rufmodus == ENDTEMPERATURAUTOMATIK) modus = ABBRUCH;
    rufquittung = false;
  }
}
//--------------------------------------------------------------------------------


// Funktion braumeisterruf--------------------------------------------------------
void funktion_braumeisterruf()      //Modus = BRAUMEISTERRUFALARM
{
  if (anfang == 0)
  {
    anfang = 1;
  }


  if (millis() >= (altsekunden + 1000))
  {
    lcd.setCursor(12, 2);
    lcd.print("        ");
    if (millis() >= (altsekunden + 1500))
      altsekunden = millis();
  }
  else
  {
    lcd.setCursor(0, 2); lcd.print("            ");
    lcd.setCursor(12, 2);
    lcd.print("weiter ?");
  }

  if (ButtonPressed)
  {

    lcd.setCursor(12, 2);
    lcd.print("        ");  //Text "weiter ?" löschen

    lcd.setCursor(0, 3);
    lcd.print("             "); //Löscht Text bei
    if (sensorfehler == 1) {
      sensorfehler = 0;  //Sensorfehler oder Alarmtest
      lcd.setCursor(8, 0);
      lcd.print("            ");
    }

    anfang = 0;
    modus = rufmodus;
    if (rufmodus == KOCHEN) {
      anfang = 1;   // Hopfen zugegeben!
      lcd.setCursor(0, 2);
      lcd.print("                    ");
    }

    delay(500);     //kurze Wartezeit, damit nicht
    //durch unbeabsichtigtes Drehen
    //der nächste Vorgabewert
    //verstellt wird
  }
}
//--------------------------------------------------------------------------------

// Funktion Anzahl Hopfengaben ---------------------------------------------------

void anzahl_hopfengaben()             //Modus = ANZAHL_HOPFENGABEN
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengaben");
    lcd.setCursor(5, 1);
    lcd.print("Anzahl");
    anfang = 1;
    drehen = hanz;
    lcd.setCursor(14, 1);
    lcd.print(hanz);
    dclick = false;
  }


  if (hanz < 9)
  {
    lcd.setCursor(14, 1); lcd.print(hanz);
  }
  else
  {
    lcd.setCursor(13, 1); lcd.print(hanz);
  }

  hanz = drehen;

  if (drehen >= maxgaben)
  {
    hanz = maxgaben; drehen = maxgaben;
  }

  if (drehen <  2)
  {
    hanz = 1; drehen = 1;
  }

  if (ButtonPressed)
  {
    modus = NACHISOZEIT;
    anfang = 0;
  }
}

//------------------------------------------------------------------------------

//Funktion Nachisomerisierung --------------------------------------------------

void nachisozeit()           //Modus = NACHISOZEIT
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Nachisomerisierung");
    lcd.setCursor(0, 1);
    lcd.print("Zeit:");
    drehen = nachisomerisierung;
    anfang = 1;
    dclick = false;
  }

  lcd.setCursor(6, 1); lcd.print(nachisomerisierung); lcd.print("  min. ");
  if (drehen > 90) drehen = 90;
  nachisomerisierung = drehen;

  if (ButtonPressed)
  {
    modus = HOPFENGABE;
    anfang = 0;
  }
}

//--------------------------------------------------------------------------------

// Funktion Hopfengabe------------------------------------------------------------

void funktion_hopfengabe()      //Modus = HOPFENGABE
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengabe:");
    lcd.setCursor(0, 1);
    lcd.print("Kochen ");
    lcd.print("f"); lcd.print(char(0xF5)); lcd.print("r");

    if (!rezept_eingelesen)  //sonst gelten die Daten aus dem Rezept
    {
      if (kochzeit >= 60)  // weniger ist eigentlich nicht sinnvoll
      {
        if (h == 0) hopfengabe[h] = kochzeit;  //Typische Zeiten für das Hopfenkochen
        if (h == 1) hopfengabe[h] = kochzeit - 20;
        if (h == 2) hopfengabe[h] = 10;
      }
    }
    drehen = hopfengabe[h];

    anfang = 1;
    dclick = false;
  }

  lcd.setCursor(11, 0); lcd.print(h + 1);

  hopfengabe[h] = drehen;
  lcd.setCursor(12, 1);

  lcd.print(drehen);


  lcd.print(" min ");

  if (hopfengabe[h] <= 0)
  {
    hopfengabe[h] = 0;
    drehen = 0;
  }
  if (hopfengabe[h] >= kochzeit)
  {
    hopfengabe[h] = kochzeit;
    drehen = kochzeit;
  }


  if (ButtonPressed)
  {
    if (h == (hanz - 1)) {
      lcd.clear();
      modus = STARTKOCHEN;

    } else {
      h++;
    }
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Funktion Kochzeit--------------------------------------------------------------

void funktion_kochzeit()      //Modus = KOCHZEIT
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Vorgaben");
    lcd.setCursor(3, 1);
    lcd.print("Kochzeit");
    lcd.setCursor(16, 1);
    lcd.print("min.");

    lcd.setCursor(2, 2);
    lcd.print("Dr"); lcd.print(char(0xF5)); lcd.print("cken = weiter");

    drehen = kochzeit;

    heizung = EIN; //Heizung einschalten
    rwerk = 1;
    umwaelzerEinAus(0);

    anfang = 1;
    dclick = false;
  }

  kochzeit = drehen;

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("  ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }

  if (ButtonPressed)
  {
    modus = ANZAHL_HOPFENGABEN; // Anzahl Hopfengaben
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Funktion Startkochzeit---------------------------------------------------------

void funktion_startkochen()      //Modus = STARTKOCHEN
{
  if ((anfang == 0) || (dclick))
  {
    anfang = 1;
    altsekunden = millis();
    abbruchkochen = 60;
    blinker = 0;
    dclick = false;
    sollwert = kschwelle;
    lcd.setCursor(0, 0); lcd.print("Ruf bei "); lcd.print(kschwelle); lcd.print(char(0xDF)); lcd.print("C"); lcd.print("  ");
  }

  if (millis() <= (altsekunden + 1000))
  {

    if ((isttemp >= kschwelle) || (schwelle_erreicht))
    {
      schwelle_erreicht = true;

      ruf = true;
      if (anfang != 3) {
        meldung_detail = "Kochschwelle-erreicht-60s-Zeit!";
        lcd.setCursor(0, 1);
        lcd.print("Abbruch in ");
        if (abbruchkochen <= 60) {
          lcd.print(abbruchkochen);
          lcd.print(" s ");

        }
      }
      lcd.setCursor(0, 2);
      if (blinker == 0)
      {
        if (anfang == 3)
        {
          lcd.print("Start Zeitz");
          lcd.print(char(0xE1));
          lcd.print("hlung?");
          meldung_detail = "Button-Kochen-Start!!";
          ruf = false;
        }
        else
        {
          lcd.print("Bitte quittieren!");
          meldung_detail = "Ruf-quittieren!";
          ruf = true;
        }
        if (anfang == 1) {
          signalton(500);
          anfang = 2;
        } else if (anfang == 2)
        { if (abbruchkochen > 61) modus = ABBRUCH;
          if (abbruchkochen % 10 == 0) signalton(500);
        }
      }
      else lcd.print("                   ");

    }
    else
    {
      if (blinker == 0)
      {
        lcd.setCursor(0, 2);
        lcd.print("Warten!");
        meldung_detail = "Bitte-warten!";
        schwelle_erreicht = false;
      }

      if (blinker == 1)
      {
        lcd.setCursor(0, 2);
        lcd.print("                   ");
      }
    }

  } else {
    if (blinker == 0) blinker = 1;
    else blinker = 0;
    altsekunden = millis();
    if (schwelle_erreicht) abbruchkochen--;
  }

  if (schwelle_erreicht)
  {
    if ((ButtonPressed) || (rufquittung))
    {
      altmodus = STARTKOCHEN;
      if (anfang == 3)
      {
        anfang = 0;
        lcd.setCursor(0, 2); lcd.print("               ");
        modus = KOCHEN;
      }
      if (anfang == 2) {
        anfang = 3;
        lcd.setCursor(0, 1);
        lcd.print("                    ");
      }

    }
  }
}
//--------------------------------------------------------------------------------

// Funktion Kochen----------------------------------------------------------------

void funktion_kochen()      //Modus = KOCHEN
{

  if ((anfang == 0) || (dclick))
  {
    ruf = false;
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Kochen");
    lcd.setCursor(3, 1);
    lcd.print("Kochzeit ");
    meldung_detail = "Kochen_gestartet";
    meldung = "41-Kochen";

    if (!dclick) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    if (zeit_setzen) {
      RastZeit_h = EEPROM.read(zeit_h);
      RastZeit_m = EEPROM.read(zeit_m);
      RastZeit_s = EEPROM.read(zeit_s);
      setTime(RastZeit_h, RastZeit_m, RastZeit_s, 00, 01, 01);
      zeit_setzen = false;
    }

    if (notfall_aktiv)
    {
      Notfallmodus = EEPROM.read(0);
      if (EEPROM.read(notfallmodus) != 1) EEPROM.write(notfallmodus, 1); //Notfallmodus aktiviert
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
      EEPROM.write(hopfengabe1, hopfengabe[0]);
      EEPROM.write(hopfengabe2, hopfengabe[1]);
      EEPROM.write(hopfengabe3, hopfengabe[2]);
      EEPROM.write(hopfengabe4, hopfengabe[3]);
      EEPROM.write(hopfengabe5, hopfengabe[4]);
      EEPROM.write(hopfengabe6, hopfengabe[5]);
      EEPROM.write(Kochzeit, kochzeit);
    }

    delay(400); //Anzeigepause

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    anfang = 1;
    if (!dclick) h = 0;  //Hopfengabengabe Zähler
    drehen = kochzeit;
    dclick = false;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("    ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }

  lcd.setCursor(12, 1); lcd.print(kochzeit); lcd.print(" min. ");

  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }

  minuten = ((stunden * 60) + minutenwert);


  if (sekunden < 10)                       //in den ersten 10 s einer Minute wird
    //der Minutenwert übertragen,
    zeit_detail = String(minuten);
  else
  {
    meldung_detail = "Kochen-gestartet";     //sonst die aktuelle Aktion
    meldung = "Kochen";
  }

  if ((kochzeit - minuten == hopfengabe[h]) && (h < hanz))
  {
    lcd.setCursor(0, 2); lcd.print(h + 1); lcd.print(". H-Gabe");
    meldung_detail = String(h + 1) + ".-Hopfengabe";
    h++;
    signalton(2000);
  }

  if ((sekunden + 10) % 30 == 0) {
    lcd.setCursor(0, 2);
    lcd.print("          ");
  }


  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(14, 0);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(14, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(13, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  kochzeit = drehen;

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (notfall_aktiv)
  {

    if (sekunden == 0)  //einmal pro Minute speichern
    {
      EEPROM.write(zeit_h, stunden);
      EEPROM.write(zeit_m, minuten);
      EEPROM.write(zeit_s, sekunden);
    }
  }

  if (minuten >= kochzeit)     //Kochzeitende
  {
    meldung_detail = "Kochzeit-beendet";
    //    modus = ABBRUCH;                //Abbruch nach Rufalarm
    rufmodus = modus;
    modus = RUFALARM;
    regelung = AUS;              //Regelung aus
    heizung = AUS;               //Heizung aus
    y = 0;
    braumeister[y] = 2;
    anfang = 0;
    if (nachisomerisierung > 0)
      modus = NACHISOMERISIERUNG; else modus = ABBRUCH;
  }

}
//--------------------------------------------------------------------------------

// Funktion Nachisomerisierung----------------------------------------------------

void nachiso()      //Modus = NACHISOMERISIERUNG
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Nachisom.");
    lcd.setCursor(0, 1);
    lcd.print("Wartezeit: ");
    lcd.setCursor(11, 1);
    lcd.print("min. ");

    if (!dclick) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    if (zeit_setzen) {
      nachiso_h = EEPROM.read(isozeit_h);
      nachiso_m = EEPROM.read(isozeit_m);
      nachiso_s = EEPROM.read(isozeit_s);
      setTime(nachiso_h, nachiso_m, nachiso_s, 00, 01, 01);
      zeit_setzen = false;
    }

    if (notfall_aktiv)
    {
      Notfallmodus = EEPROM.read(0);
      if (EEPROM.read(notfallmodus) != 1) EEPROM.write(notfallmodus, 1); //Notfallmodus aktiviert
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
      EEPROM.write(niso, nachisomerisierung);
    }

    delay(400); //Anzeigepause

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    anfang = 1;
    drehen = nachisomerisierung;
    dclick = false;
  }

  zeit_detail = String(minuten);

  if (nachisomerisierung < 100)
  {
    lcd.setCursor(11, 1);
    lcd.print("  ");
    lcd.print(nachisomerisierung); lcd.print(" min. ");
  }
  else
  {
    lcd.setCursor(11, 1);
    lcd.print(" ");
    lcd.print(nachisomerisierung); lcd.print(" min. ");
  }

  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }

  minuten = ((stunden * 60) + minutenwert);

  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(14, 0);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(14, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(13, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  //  meldung_detail = ":" + String(minuten);
  if (drehen > 90) drehen = 90;
  nachisomerisierung = drehen;

  if (notfall_aktiv)
  {

    if (sekunden == 0)  //einmal pro Minute speichern
    {
      EEPROM.write(isozeit_h, stunden);
      EEPROM.write(isozeit_m, minuten);
      EEPROM.write(isozeit_s, sekunden);
    }
  }

  if ((minuten >= nachisomerisierung) || (isttemp <= isoTemperatur))    //Nachisomerisierung Ende
  {
    meldung_detail = "Nachiso-beendet";
    modus = ABBRUCH;                //Abbruch nach Rufalarm
    rufmodus = modus;
    modus = RUFALARM;
    y = 0;
    braumeister[y] = 2;
    anfang = 0;
  }

}
//--------------------------------------------------------------------------------

//Funktion Gradientenfaktor HEIZEN eingeben --------------------------------------

void khwert()            //Modus = GRADIENTENFAKTOR_HEIZEN
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Faktor Heizen");
    lcd.setCursor(0, 1);
    lcd.print("kh = ");

    kh =  EEPROMReadlong(graf_h);
    anfang = 1;
    drehen = kh;
    dclick = false;
  }

  if (drehen < 0) drehen = 200;
  if (drehen > 200) drehen = 0;


  kh  = drehen;
  lcd.setCursor(5, 1); lcd.print(kh / 10); lcd.print(' ');

  if (ButtonPressed)
  {
    if (EEPROMReadlong(graf_h) != drehen) EEPROMWritelong(graf_h, drehen);
    speichern(2);
    modus = EINSCHALTVERZOEGERUNG_HEIZRELAIS;
    kh /= 10;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

//Funktion Einschaltverzögerung HEIZEN eingeben ----------------------------------

void ESVHeiz()           //Modus = EINSCHALTVERZOEGERUNG_HEIZRELAIS
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Einschaltv. Heizen");
    lcd.setCursor(0, 1);
    lcd.print("ESVH = ");

    esvheizen =  EEPROMReadlong(wein_h);
    anfang = 1;
    drehen = esvheizen / 1000;
  }

  if (drehen < 0) drehen = 60;
  if (drehen > 60) drehen = 0;

  lcd.setCursor(7, 1); lcd.print(drehen); lcd.print(" s ");

  esvheizen = drehen * 1000L;

  if (ButtonPressed)

  {
    if (EEPROMReadlong(wein_h) != esvheizen) EEPROMWritelong(wein_h, esvheizen);
    speichern(2);
    modus = GRADIENTENFAKTOR_KUEHLEN;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

//Funktion Gradientenfaktor KÜHLEN eingeben --------------------------------------

void kkwert() //Modus = GRADIENTENFAKTOR_KUEHLEN
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Faktor Kuehlen");
    lcd.setCursor(0, 1);
    lcd.print("kk = ");

    kk =  EEPROMReadlong(graf_k);
    anfang = 1;
    drehen = kk;
  }

  if (drehen < 0) drehen = 200;
  if (drehen > 200) drehen = 0;


  kk  = drehen;
  lcd.setCursor(5, 1); lcd.print(kk / 10); lcd.print(' ');

  if (ButtonPressed)
  {
    if (EEPROMReadlong(graf_k) != drehen) EEPROMWritelong(graf_k, drehen);
    speichern(2);
    modus = WARTEZEIT_KUEHLEN;
    kk /= 10;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Funktion Einschaltverzögerung Kühlen (Kompressorschutz)------------------------

void wartenKuehlen()  //Modus = WARTEZEIT_KUEHLEN
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Wartezeit Kuehlen");
    lcd.setCursor(0, 1);
    lcd.print("ESVK = ");
    kzeitverz = EEPROMReadlong(kuehlverz);
    kzeitverz = kzeitverz / (60000L);
    drehen = kzeitverz;
    anfang = 1;
  }
  else
  {

    kzeitverz = drehen;


    lcd.setCursor(7, 1); lcd.print(kzeitverz); lcd.print(" min  ");

    if (ButtonPressed)
    {
      if (EEPROMReadlong(kuehlverz) != kzeitverz * 60000L) EEPROMWritelong(kuehlverz, kzeitverz * 60000L);
      kzeitverz = kzeitverz * 60000L;
      speichern(2);
      modus = SETUPMENUE;
      anfang = 0;
    }
  }
}
//-------------------------------------------------------------------------------------

//Funktion Logiing EIN-AUS ------------------------------------------------------------

void LoggingEinAus() //Modus = LOGGING_EIN_AUS
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Logging Ein?");
    drehen = EEPROM.read(logging);
    anfang = 1;
    dclick = false;
  }

  if (drehen > 1) drehen = 0;
  if (drehen < 0) drehen = 1;

  lcd.setCursor(0, 1); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  loggen = drehen;


  if (ButtonPressed)
  {
    modus = SETUPMENUE;
    if (EEPROM.read(logging) != drehen) EEPROM.write(logging, drehen);
    if (loggen) {
      modus = LOGWAHL;
    }
    speichern(2);
    anfang = 0;
  }
}
//-------------------------------------------------------------------------------

//Funktion Auswahl des loggingprogramms -----------------------------------------

void Logwahl() //Modus = LOGWAHL
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Loggerwahl");
    lcd.setCursor(0, 1);
    lcd.print("SerialCom:");
    lcd.setCursor(0, 2);
    lcd.print("LogView  :");


    drehen = EEPROM.read(loggerwahl);
    anfang = 1;
    dclick = false;
  }

  if (drehen > 2) drehen = 0;
  if (drehen < 0) drehen = 1;

  if (drehen == 0) {
    lcd.setCursor(11, 1);
    lcd.print("<-");
    lcd.setCursor(11, 2);
    lcd.print("  ");
    lcd.setCursor(18, 1);
    lcd.print("  ");
  }
  if (drehen == 1) {
    lcd.setCursor(11, 1);
    lcd.print("  ");
    lcd.setCursor(11, 2);
    lcd.print("<-");
    lcd.setCursor(18, 1);
    lcd.print("  ");
  }

  loggen = drehen;

  if (ButtonPressed)
  {
    modus = SETUPMENUE;
    if (EEPROM.read(loggerwahl) != drehen) EEPROM.write(loggerwahl, drehen);
    switch (drehen)
    {
      case 0 : {
          serialcom = true;
          logview = false;
          break;
        }
      case 1 : {
          logview = true;
          serialcom = false;
          break;
        }
    }
    speichern(2);
    anfang = 0;
  }
}
//-------------------------------------------------------------------------------------

//FUNKTION WLAN-Steuerung EIN-AUS -----------------------------------------------------

void WLANSteuerungAktiv() //Modus = WLANSTEUERUNG_AKTIV
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("KBH2AiO aktiv?");
    drehen = EEPROM.read(Wlan);
    anfang = 1;
    dclick = false;
  }

  drehen %= 2;

  lcd.setCursor(15, 0); if (drehen == 1) lcd.print("J"); else lcd.print("N");


  if (ButtonPressed)
  {
    modus = EXTERNMENUE;
    byte alterWert = EEPROM.read(Wlan);
    if (alterWert != drehen) EEPROM.write(Wlan, drehen);
    speichern(2);
    if (drehen == 0) wlan_steuerung = false; else {
      if (alterWert == 0)
      {
        wlan_steuerung = true;
        Serial2.begin(115200);
        connectWiFi();
      }
    }
    anfang = 0;
  }
}
//-------------------------------------------------------------------------------------

//Funktion Rührer oder Pumpe ?

void auswahl_umwaelzer() //Modus = AUSWAHL_UMWAELZER
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrer oder Pumpe?");
    lcd.setCursor(0, 1);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrer  :");
    lcd.setCursor(0, 2);
    lcd.print("Pumpe   :");

    drehen = EEPROM.read(umwaelzwahl);

    if (drehen == 0) {
      lcd.setCursor(11, 1);
      lcd.print("<-");
      lcd.setCursor(11, 2);
      lcd.print("  ");
    }

    if (drehen == 1) {
      lcd.setCursor(11, 1);
      lcd.print("  ");
      lcd.setCursor(11, 2);
      lcd.print("<-");
    }
    anfang = 1;
    dclick = false;
  }

  if (drehen > 1) drehen = 0;
  if (drehen < 0) drehen = 1;

  if (drehen == 0) {
    lcd.setCursor(11, 1);
    lcd.print("<-");
    lcd.setCursor(11, 2);
    lcd.print("  ");

  }
  if (drehen == 1) {
    lcd.setCursor(11, 1);
    lcd.print("  ");
    lcd.setCursor(11, 2);
    lcd.print("<-");
  }

  umwaelzen = drehen;


  if (ButtonPressed)
  {
    if (EEPROM.read(umwaelzwahl) != drehen) EEPROM.write(umwaelzwahl, drehen);
    switch (drehen)
    {
      case 0 : {
          ruehrer = true;
          pumpe = false;
          break;
        }
      case 1 : {
          ruehrer = false;
          pumpe = true;
          break;
        }
    }

    speichern(2);
    anfang = 0;
    modus = UMWAELZER_DAUERLAUF;
  }
}
//-------------------------------------------------------------------------------------

//Funktion Rührer/Pumpe aktiv beim Einmaischen

void RuePumAktiv() //Modus = RUEPUMAKTIV ------------------------------------------------
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrer aktiv");
    lcd.setCursor(0, 1); lcd.print("beim Einmaischen?");
    drehen = EEPROM.read(ruepumeinmaisch);
    anfang = 1;
    dclick = false;
  }

  drehen %= 2;
  lcd.setCursor(18, 1); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  ruehrer_beim_einmaischen = drehen;

  if (ButtonPressed)
  {
    modus = SETUPMENUE;
    if (EEPROM.read(ruepumeinmaisch) != drehen) EEPROM.write(ruepumeinmaisch, drehen);
    speichern(3);
    anfang = 0;
  }
}
//-------------------------------------------------------------------------------------

//Funktion Notfallmodus EIN-AUS

void StromAus() //Modus = NOTFALLMODUS ------------------------------------------------
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Notfall Ein?");
    drehen = EEPROM.read(notfall);
    anfang = 1;
    dclick = false;
  }

  drehen %= 2;

  lcd.setCursor(0, 1); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  notfall_aktiv = drehen;

  if (ButtonPressed)
  {
    modus = SETUPMENUE;
    if (EEPROM.read(notfall) != drehen) EEPROM.write(notfall, drehen);
    speichern(2);
    anfang = 0;
  }
}
//-------------------------------------------------------------------------------------

//Funktion Verwendungszweck der 3. Funksteckdose --------------------------------------

void steckdosenwahl() //Modus = DRITTE_FUNKSTECKDOSE
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Funkruf?");
    lcd.setCursor(0, 1);
    lcd.print("sonst Nachguss!");
    drehen = EEPROM.read(dritte_funksteckdose);
    lcd.setCursor(10, 0); if (drehen == 1) lcd.print("J"); else lcd.print("N");
    anfang = 1;
    dclick = false;
  }

  drehen %= 2;

  lcd.setCursor(10, 0); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  if (ButtonPressed)
  {
    modus = FRAGESTECKDOSENCODEEINGEBEN;
    if (EEPROM.read(dritte_funksteckdose) != drehen) EEPROM.write(dritte_funksteckdose, drehen);
    speichern(2);
    if (drehen == 1) {
      funkruf = true;
    } else {
      funkruf = false;
    }
    anfang = 0;

  }
}
//-------------------------------------------------------------------------------------

//Funktion Steckdosencode anschliessend direkt eingeben? ------------------------------

void frageCodeEingeben() //Modus = FRAGESTECKDOSENCODEEINGEBEN
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Steckdosencode");
    lcd.setCursor(0, 1);
    lcd.print("eingeben?");
    drehen = 1;
    anfang = 1;
    dclick = false;
  }

  drehen %= 2;

  lcd.setCursor(10, 1); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  if (ButtonPressed)
  {

    anfang = 0;
    if (drehen == 0) modus = SETUPMENUE; else modus = STECKDOSENCODE;
  }
}
//-------------------------------------------------------------------------------------

//Funktion WLAN-Verbinden -------------------------------------------------------------


void connectWiFi()   //Modus = CONNECT_WIFI
{
  lcd.clear();
  pinMode(reset8266, OUTPUT);
  digitalWrite(reset8266, LOW);
  lcd.setCursor(0, 0); lcd.print("Init ESP8266");
  digitalWrite(reset8266, HIGH);                        //Hardware Reset
  delay(2000);
  if (Serial2.available())
  {
    //    delay(2000);
    Serial2.print("AT+RST\r\n");                          //Software Reset
    getReply( 1000 );
    Serial2.print("AT+CIOBAUD=115200\r\n");               //Einstellen der Baudrate
    getReply( 1000 );

#ifdef dbg
    Serial.println("Start\r\n\r\n");
#endif
    lcd.setCursor(0, 1); lcd.print("Start WLAN");
    lcd.setCursor(0, 2); lcd.print("Param. UPD ESP8266");
#ifdef dbg
    Serial.println("Modul Reset");
#endif
    Serial2.print("AT+RST\r\n");                          //Software Reset
    getReply( 1000 );
#ifdef dbg
    Serial.println("Firmware");
#endif
    Serial2.print("AT+GMR\r\n");                          //Ausgabe der Firmware
    getReply( 1000 );


    strcpy(command, "AT+CWSAP=");                         //Zugangsdaten und Verschlüsselung
    strcat(command, "\"");
    strcat(command, "AiO-LAN");
    strcat(command, "\"");
    strcat(command, ",");
    strcat(command, "\"");
    strcat(command, "Hobbybrauer");
    strcat(command, "\"");
    strcat(command, ",4,3");                              //Channel 4 und WPA2_PSK
    strcat(command, "\r\n");
    Serial2.print(command);
#ifdef dbg
    Serial.println(command);
#endif
    getReply( 1000 );

    Serial2.print("AT+CWSAP?\r\n");                       //wurden die Werte übernommen?
    getReply( 2000 );

    strcpy(command, "AT+CIPAP=");                         //Setzen der IP-Adresse des ESP
    strcat(command, "\"");
    strcat(command, "192.168.100.1");
    strcat(command, "\"");
    strcat(command, "\r\n");
    Serial2.print(command);
#ifdef dbg
    Serial.println(command);
#endif
    getReply( 3000 );


#ifdef dbg
    Serial.println("IP-Adresse holen");
#endif
    Serial2.print("AT+CIFSR\r\n");
    getReply( 2000 );

    Serial2.println("AT+CIPMUX=1");                       //Multiple Connections
    getReply( 1500 );

#ifdef dbg
    Serial.println("Server einrichten");
#endif
    Serial2.print("AT+CIPSERVER=1\r\n");
    getReply( 2000 );

    strcpy(command, "AT+CIPSTART=4,");
    strcat(command, "\"");
    strcat(command, "UDP");
    strcat(command, "\"");
    strcat(command, ",");
    strcat(command, "\"");
    strcat(command, "192.168.100.255");
    strcat(command, "\"");
    strcat(command, ",50000,50000,0");
    strcat(command, "\r\n");
    Serial2.print(command);
#ifdef dbg
    Serial.println(command);
#endif
    getReply( 3000 );

  }

  if (modus == CONNECT_WIFI) modus = SETUPMENUE;
}


//-------------------------------------------------------------------------------

//Funktion Daten vom Empfänger --------------------------------------------------

void getReply(int wait)  //kein separater Modus
{

  int tempPos = 0;
  long int time = millis();
  while ( (time + wait) > millis())
  {
    while (Serial2.available())
    {
      char c = Serial2.read();
      if (tempPos < 500) {
        reply[tempPos] = c;
        tempPos++;
      }
    }
    reply[tempPos] = 0;
  }

#ifdef dbg
  if (printReply) {
    Serial.println( reply );
    //   Serial.println(line);
  }
#endif

}
//-------------------------------------------------------------------------------

//WLAN EIN-AUS ------------------------------------------------------------------

void wifieinaus() //Modus = WIFI_EIN_AUS
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("WLAN:");
    lcd.setCursor(0, 1);
    lcd.print("EIN:");
    anfang = 1;
    drehen = EEPROM.read(Wlan);
  }

  char antwort = ja_nein[drehen % 2];

  lcd.setCursor(5, 1); lcd.print(antwort);

  if (ButtonPressed)
  {
    modus = SETUPMENUE;
    if (EEPROM.read(Wlan) != drehen % 2) EEPROM.write(Wlan, drehen % 2);
    speichern(2);
    anfang = 0;
    if (drehen % 2 == 1) {
      Serial2.begin(115200);
      wlanAktiv = true;
      modus = CONNECT_WIFI;
    } else Serial2.end();

  }
}

//-------------------------------------------------------------------------------------


//LogView Studio -- Download unter : http://www.logview.info/forum/index.php?resources/
//hier können natürlich auch andere Protokolle hinterlegt werden!

void logger_logview() //Daten für Logview-Studio bereitstellen ------------------------
{
  if (loggen)
  {
    String data;

    data = "$";
    data += String(sollwert);
    data += ";";
    data += String(isttemp);
    data += ("\r\n");

    Serial.print(data);                //Sende Messdaten via COM

    Serial2.print("AT+CIPSEND=4,");    //Sende Messdaten via UDP

    Serial2.println(data.length() + 2);

    delay(50);
    if (Serial2.find('>')) {  //ESP8266 sendebereit
      Serial2.print(data);  //abschicken
      Serial2.print("\r\n");
      //     delay(100);
    }
  }
}

//-------------------------------------------------------------------------------------

//SerialComInstruments - Download unter: http://www.serialcominstruments.com/instrument4.php
//hier können natürlich auch andere Protokolle hinterlegt werden!


void logger_serial()  //Daten für SerialComInstruments4 bereitstellen
{
  String data;
  data = "#";
  data += "01";  //Kanal 1
  data += "M";
  data += String(isttemp);  //Isttemperatur
  data += "<";

  data += "#";
  data += "02";  //Kanal 2
  data += "M";
  data += String(sollwert);  //Solltemperatur
  data += "<";

  data += "#";
  data += "03"; //Kanal 3
  data += "M";
  data += String(gradient);  //Gradient
  data += "<";

  if (heizung == EIN)
  {
    data += "#";
    data += "04"; //Kanal 4
    data += "M";
    data += "1";  //Heizung EIN
    data += "<";
  }
  else
  {
    data += "#";
    data += "04";  //Kanal 4
    data += "M";
    data += "0";  //Heizung AUS
    data += "<";
  }

  if (rwerk == 1)
  {
    data += "#";
    data += "05";  //Kanal 5
    data += "M";
    data += "1";  //Umwaelz EIN
    data += "<";
  }
  else
  {
    data += "#";
    data += "05";  //Kanal 5
    data += "M";
    data += "0";  //Umwaelz AUS
    data += "<";
  }

  Serial.println(data); //Com-Port-Datenübertragung

  if (wlanAktiv) delay(100);

  Serial2.print("AT+CIPSEND=4,");    //Sende Messdaten

  Serial2.println(data.length() + 2);

  delay(50);
  if (Serial2.find('>')) {  //ESP8266 sendebereit
    Serial2.print(data);  //abschicken
    Serial2.print("\r\n");
    //    delay(100);
  }
}

//-------------------------------------------------------------------------------------

//Funktion kurzen Signalton abgeben ---------------------------------------------------

void signalton(int dauer)  //kein separater Modus
{
  RufEIN();
  delay(dauer);
  RufAUS();
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
void EEPROMWritelong(int address, long value)
{
  byte four = (value & 0xFF);
  byte three = ((value >> 8) & 0xFF);
  byte two = ((value >> 16) & 0xFF);

  EEPROM.write(address, four);
  EEPROM.write(address + 1, three);
  EEPROM.write(address + 2, two);
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
long EEPROMReadlong(long address)
{
  long four = EEPROM.read(address);
  long three = EEPROM.read(address + 1);
  long two = EEPROM.read(address + 2);

  return ((four << 0) & 0xFF) + ((three << 8) & 0xFFFF) + ((two << 16) & 0xFFFFFF);
}
//-------------------------------------------------------------------------------------

//Funkcodes einlesen ------------------------------------------------------------------

void init_funk()
{
  funk_heizung_ein = EEPROMReadlong(Heizung_EIN);
  funk_heizung_aus = EEPROMReadlong(Heizung_AUS);
  funk_umwaelzer_ein = EEPROMReadlong(Umwaelz_EIN);
  funk_umwaelzer_aus = EEPROMReadlong(Umwaelz_AUS);
  funk_variabel_ein = EEPROMReadlong(Nachguss_EIN);
  funk_variabel_aus = EEPROMReadlong(Nachguss_AUS);
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
void speichern(byte zeile)
{
  lcd.setCursor(0, zeile);
  lcd.print("speichern");
  delay(500);
}
//-------------------------------------------------------------------------------------

// Funktion Eingabe der Rastanzahl------------------------------------------------
void funktion_gaerrastanzahl()          //Modus = GAERFUEHRUNG
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(8, 1);
    lcd.print("G"); lcd.print(char(0xE1)); lcd.print("rrasten");

    drehen = gaerrasten;
    anfang = 1;
    dclick = false;
  }



  gaerrasten = drehen;


  if (gaerrasten <= 1)
  {
    gaerrasten = 1;
    drehen = 1;
  }
  if (gaerrasten >= 5)
  {
    gaerrasten = 5;
    drehen = 5;
  }

  lcd.setCursor(19, 1);
  lcd.print(gaerrasten);

  if (ButtonPressed)
  {
    modus = GAERSTARTTEMPERATUR;
    anfang = 0;
  }
}
//------------------------------------------------------------------------------------

// Funktion Gärstarttemperatur--------------------------------------------------------
void gaerstarttemperatur()      //Modus = GAERSTARTTEMPERATUR
{
  if ((anfang == 0) || (dclick))
  {
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(0, 1);
    lcd.print("G"); lcd.print(char(0xE1)); lcd.print("rung Start:     ");
    drehen = gaerstart;
    first = true;
    anfang = 1;
    dclick = false;
  }

  gaerstart = drehen;

  if (gaerstart <= 0)
    gaerstart = 30;

  if (gaerstart > 30)
    gaerstart = 0;

  lcd.setCursor(16, 1);
  lcd.print(gaerstart);
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus = GAERTEMPERATUREN;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Wechsel Zeit-Temperatureingabe Gären-------------------------------------------

void wechsel_gaer_temp_zeit() //Modus = WECHSEL_GAER_TEMP_ZEIT
{
  if (x < gaerrasten)
  {
    x++;
    modus = GAERTEMPERATUREN;           //Sprung zur Rasttemperatureingabe
    anfang = 0;
  }
  else {
    x = 1;
    modus =  ANSTELLTEMPERATUR;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------


// Funktion Rasteingabe Temperatur--------------------------------------------------
void gaerrasteingabe()      //Modus = GAERTEMPERATUREN
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = gaerRastTemp[x];
    anfang = 1;
    dclick = false;
  }

  gaerRastTemp[x] = drehen;

  if (gaerRastTemp[x] < 0)
  {
    gaerRastTemp[x] = 0;
    drehen = 0;
  }
  if (gaerRastTemp[x] > 30)
  {
    gaerRastTemp[x] = 30;
    drehen = 30;
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");
  lcd.setCursor(14, 1);
  lcd.print(int(gaerRastTemp[x]));
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus = GAERZEITEN;
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------

// Funktion Rasteingabe Zeit--------------------------------------------------------
void gaerzeiteingabe()      //Modus = GAERZEITEN
{

  if ((anfang == 0) || (dclick))
  {
    drehen = gaerRastZeit[x];
    anfang = 1;

    lcd.setCursor(0, 0);
    lcd.print("Eingabe Std.");

    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");
    lcd.setCursor(14, 1);
    lcd.print(int(gaerRastTemp[x]));
    lcd.print(char(0xDF)); lcd.print("C ");


    dclick = false;
  }

  gaerRastZeit[x] = drehen;

  if (gaerRastZeit[x] <= 1)
  {
    gaerRastZeit[x] = 1;
    drehen = 1;
  }
  if (gaerRastZeit[x] == 254)
  {
    gaerRastZeit[x] = 255;
    drehen = 255;
  }

  if (gaerRastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(gaerRastZeit[x]));
    lcd.print(" h");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(gaerRastZeit[x]));
    lcd.print(" h");
  }


  if (ButtonPressed)
  {
    modus = WECHSEL_GAER_TEMP_ZEIT;
    anfang = 0;
  }
}
//------------------------------------------------------------------------------------

// Funktion Anlauf Anstelltemperatur--------------------------------------------------
void anlauf_anstelltemperatur()      //Modus = ANSTELLTEMPERATUR
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Anstelltemperatur");
    lcd.setCursor(9, 1);
    lcd.print("Soll:");
    drehen = gaerstart;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
    if (notfall_aktiv) EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
  }
  if (drehen > 30) drehen = 0;
  gaerstart = drehen;

  sollwert = gaerstart;

  lcd.setCursor(15, 1);
  lcd.print(sollwert); lcd.print(char(0xDF)); lcd.print("C ");

  if ((isttemp >= sollwert - gaerhysterese) && (isttemp <= sollwert + gaerhysterese)) // Sollwert erreicht ?
  {
    modus = ANLAUFGAERSTUFEN;
    anfang = 0;
    meldung_detail = "Anstelltemperatur_erreicht";
  }
}
//-------------------------------------------------------------------------------------

// Funktion Anlauf Gärstufen--------------------------------------------------
void AnlaufGaerstufen()      //Modus = ANLAUFGAERSTUFEN
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Anlauf ");
    lcd.print(x);
    lcd.print(".G"); lcd.print(char(0xE1)); lcd.print("rrast");
    lcd.setCursor(9, 1); lcd.print("soll: ");
    meldung_detail = "Anl._" + String(x) + "._Gaerstufe";
    drehen = gaerRastTemp[x];
    anfang = 1;
    first = true;
    dclick = false;
    if (notfall_aktiv)
    {
      EEPROMWritelong(rastmodus, modus); //für Notfall Modus merken
      EEPROM.write(rastnr, x);
      EEPROM.write(rasttemp_aktuell, gaerRastTemp[x]);
      EEPROM.write(rastzeit_aktuell, gaerRastZeit[x]);
    }
  }
  if (drehen > 30) drehen = 30;
  gaerRastTemp[x] = drehen;

  sollwert = gaerRastTemp[x];

  lcd.setCursor(15, 1); lcd.print(sollwert); lcd.print(char(0xDF)); lcd.print("C ");

  if ((isttemp >= sollwert - gaerhysterese) && (isttemp <= sollwert + gaerhysterese)) // Sollwert erreicht ?
  {
    modus = GAERZEITAUTOMATIK;
    anfang = 0;
    meldung_detail = "Gaertemperatur_erreicht";
  }
}
//-------------------------------------------------------------------------------------

// Funktion Gaerzeitautomatik----------------------------------------------------------
void gaerzeitautomatik()      //Modus = GAERZEITAUTOMATIK
{
  if ((anfang == 0) || (dclick))
  { lcd.clear();
    drehen = gaerRastZeit[x];              //Zuordnung für Encoder
    lcd.setCursor(9, 1); lcd.print("soll: ");
    lcd.print(sollwert); lcd.print(char(0xDF)); lcd.print("C ");
    meldung_detail = "Gaerzeitautomatik";
    if (altx != x) {
      zeitz = true;
    }

    if (notfall_aktiv)
    {
      Notfallmodus = EEPROM.read(0);
      if (EEPROM.read(notfallmodus) != 1) EEPROM.write(notfallmodus, 1); //Notfallmodus aktiviert
      EEPROM.write(Rasten, gaerrasten);
      EEPROM.write(MaischTemp, gaerstart);

      switch (gaerrasten) {
        case 1: {
            EEPROM.write(rastzeit_1, gaerRastZeit[1]);
            EEPROM.write(rasttemp_1, gaerRastTemp[1]);
          }

        case 2: {
            EEPROM.write(rastzeit_1, gaerRastZeit[1]);
            EEPROM.write(rasttemp_1, gaerRastTemp[1]);
            EEPROM.write(rastzeit_2, gaerRastZeit[2]);
            EEPROM.write(rasttemp_2, gaerRastTemp[2]);
          }

        case 3: {
            EEPROM.write(rastzeit_1, gaerRastZeit[1]);
            EEPROM.write(rasttemp_1, gaerRastTemp[1]);
            EEPROM.write(rastzeit_2, gaerRastZeit[2]);
            EEPROM.write(rasttemp_2, gaerRastTemp[2]);
            EEPROM.write(rastzeit_3, gaerRastZeit[3]);
            EEPROM.write(rasttemp_3, gaerRastTemp[3]);
          }

        case 4: {
            EEPROM.write(rastzeit_1, gaerRastZeit[1]);
            EEPROM.write(rasttemp_1, gaerRastTemp[1]);
            EEPROM.write(rastzeit_2, gaerRastZeit[2]);
            EEPROM.write(rasttemp_2, gaerRastTemp[2]);
            EEPROM.write(rastzeit_3, gaerRastZeit[3]);
            EEPROM.write(rasttemp_3, gaerRastTemp[3]);
            EEPROM.write(rastzeit_4, gaerRastZeit[4]);
            EEPROM.write(rasttemp_4, gaerRastTemp[4]);
          }
      }
    }
  }
  altx = x;
  lcd.setCursor(0, 0);
  lcd.print(x);
  lcd.print(".G"); lcd.print(char(0xE1)); lcd.print("rrast");


  if (gaerRastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(gaerRastZeit[x]));
    lcd.print(" h");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(gaerRastZeit[x]));
    lcd.print(" h");
  }



  // Zeitzählung------------------------------------------------------------------
  if (anfang == 0)
  {
    setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    if (zeit_setzen) {
      RastZeit_h = EEPROM.read(zeit_h);
      RastZeit_m = EEPROM.read(zeit_m);
      RastZeit_s = EEPROM.read(zeit_s);
      setTime(RastZeit_h, RastZeit_m, RastZeit_s, 00, 01, 01);
      zeit_setzen = false;
    }

    lcd.setCursor(0, 2);
    lcd.print("        ");
    anfang = 1;

  }

  sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
  minuten = minute(); //aktuell Minute abspeichern für die Zeitrechnung
  stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung



  lcd.setCursor(0, 2);

  if (minuten < 10)
  {
    if (stunden == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:0"); lcd.print(minuten);
    }
  }
  else
  {
    if (stunden == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:"); lcd.print(minuten);
    }
  }


  if ((stunden < 10) && (stunden > 0))
  {
    lcd.setCursor(1, 2);
    lcd.print("0"); lcd.print(stunden); lcd.print(":");
    if (minuten < 10) {
      lcd.print("0");
      lcd.print(minuten);
    } else lcd.print(minuten);
  }

  if ((stunden >= 10) && (stunden < 100))
  {
    lcd.setCursor(1, 2);
    lcd.print(stunden); lcd.print(":");
    if (minuten < 10) {
      lcd.print("0");
      lcd.print(minuten);
    } else lcd.print(minuten);
  }

  if (stunden >= 100)
  {
    lcd.setCursor(0, 2);
    lcd.print(stunden); lcd.print(":");
    if (minuten < 10) {
      lcd.print("0");
      lcd.print(minuten);
    } else lcd.print(minuten);
  }

  String m;
  String s;
  m = String(minutenwert);
  s = String(stunden);
  if (minutenwert < 10) m = "0" + String(minutenwert);   //Aktuelle Zeit zum PC
  if (stunden < 10) s = "0" + String(stunden);
  zeit_detail = s + ":" + m;

  // Ende Zeitzählung---------------------------------------------------------------

  //Zustand für den Fall von Stromausfall speichern---------------------------------

  if (notfall_aktiv)
  {
    if (sekunden == 0)  //einmal pro Minute speichern
    {
      EEPROM.write(rastnr, x);
      EEPROM.write(rasttemp_aktuell, gaerRastTemp[x]);
      EEPROM.write(rastzeit_aktuell, gaerRastZeit[x]);
      EEPROM.write(zeit_h, stunden);
      EEPROM.write(zeit_m, minuten);
      EEPROM.write(zeit_s, sekunden);
    }
  }

  //--------------------------------------------------------------------------------

  if (drehen > 100) drehen = 0;
  gaerRastZeit[x] = drehen;   //Encoderzuordnung

  if (stunden >= gaerRastZeit[x])  // Sollwert erreicht ?
  {
    anfang = 0;
    y = x;

    if (x < gaerrasten)
    {
      modus = ANLAUFGAERSTUFEN;   // zur Temperaturregelung
      x++;                        // nächste Stufe
    } else {
      anfang = 0;
      modus = HAUPTMENUE;
    }
  }
}
//--------------------------------------------------------------------------------

void sound_einstellen()
{
  if (anfang == 0)
  {
    anfang = 1;
//    B_EIN = EEPROM.read(sound);
//    drehen = B_EIN;
    lcd.setCursor(0, 0); lcd.print("Ton: ");
  }
  drehen %= 256;
  lcd.setCursor(5, 0); lcd.print(drehen); lcd.print("     ");
//  B_EIN = drehen;
  RufEIN();

  if (ButtonPressed)
  {
    speichern(2);
//    if (EEPROM.read(sound) != B_EIN) EEPROM.write(sound, B_EIN);
    RufAUS();
    modus = SETUPMENUE;
    anfang = 0;
  }



}
// Funktion Abbruch--------------------------------------------------------------
void funktion_abbruch()       // Modus = ABBRUCH
{
  regelung = AUS;
  heizung = AUS;

  if (rwerk == 1) umwaelzerEinAus(0);
  rwerk = 0;
  HeizungAUS();
  NachgussAUS();
  RufAUS();
  ruf = false;

  UmwaelzerAUS();
  WuerzepumpeEinAus(0);

  mySwitch.send(funk_variabel_aus, 24);  //unabhängig vom Einsatzzweck AUS

  x = 1;                              //Rasteingaben
  h = 0;                              //Hopfeneingaben
  y = 1;
  n = 0;
  nachgussruf = false;
  loggen = EEPROM.read(logging);     //aktuellen Status von loggen setzen
  pause = 0;
  schwelle_erreicht = false;         // für Kochbeginn Meldung

  first = true;
  kzeitverz = 0;                    //Kompressorverzögerung

  zeitz = true;                     // Zeit läuft nach Doppelclick weiter wenn zeitz = false

  anfang = 0;                       //Daten zurücksetzen
  rufmodus = ABBRUCH;                     //bleiben erhalten
  drehen = sollwert;                  //Zuweisung für Funktion Temperaturregelung
  sollwert = 20;

  if (EEPROM.read(notfallmodus) == 1)
  {
    EEPROM.write(notfallmodus, 0); //Notfallmodus deaktiviert
    EEPROM.write(zeit_h, 0);
    EEPROM.write(zeit_m, 0);
    EEPROM.write(zeit_s, 0);
  }

  if (altmodus == KUEHLEN)
  {
    if (knot) knot = false; else knot = true;
    if (!knot) {
      if (EEPROM.read(k_aktiv) != 1) EEPROM.write(k_aktiv, 1);  //Beim Neustart in Modus 0 , bei Stromausfall in Modus (1) Kühlen
      if (EEPROM.read( g_kuehltemp) != drehen) EEPROM.write( g_kuehltemp, drehen);
      lcd.setCursor(0, 3);
      lcd.print("NOT->EIN"); meldung_detail = "NOT->EIN";
      delay(1000);
    }
    if (knot) {
      lcd.setCursor(0, 3); lcd.print("NOT->AUS"); delay(1000); //Notstart zurücksetzen
      meldung_detail = "NOT->AUS";
      if (EEPROM.read(k_aktiv) == 1) EEPROM.write(k_aktiv, 0);
    }  //nach Notfall- wieder Normalmodus
  }

  modus = HAUPTMENUE;                        //Hauptmenue
  rufmodus = HAUPTMENUE;
  altmodus = HAUPTMENUE;
  sondermodus = HAUPTMENUE;

  rezept_eingelesen = false;

  lcd.clear();
}

//---------------------------------------------------------------------------------

//Funktion UDP zum PC (KBH2AiO) ---------------------------------------------------

void zum_pc()
{
  String data;
  //0
  data = String(isttemp);  //Isttemperatur
  data += ";";
  //1
  data += String(sollwert);  //Solltemperatur
  data += ";";
  //2
  data += String(gradient);  //Gradient
  data += ";";
  //3
  if (heizung == EIN) data += "1"; else data += "0";
  data += ";";
  //4
  if (rwerk == 0) data += "0"; else data += "1";
  data += ";";
  //5
  data += modus;
  data += ";";
  //6
  data += meldung;
  data += ";";
  //7
  data += meldung_detail;
  data += ";";
  //8
#ifdef Tempsensor433
  if (anzahlTempsensoren == 1)
    data += String(nachgusssensor);
#else
  if (anzahlTempsensoren == 1)
    data += "NC";
  else
    data += String(nachgusssensor) ;
#endif
  data += ";";
  //9
  if (nachgussheizung == EIN) data += "1"; else data += "0";
  data += ";";
  //10
  if (ruf) data += "Ruf";
  data += ";";
  //11
  data += String(rastZeit[x]);
  data += ";";
  //12
  data += zeit_detail;
  data += ";";
  //13
  data += String(nachisomerisierung);
  data += ";";
  //14
  data += String(gaerRastZeit[x]);
  data += ";";




  Serial.println(data); //Com-Port-Datenübertragung

  Serial2.print("AT+CIPSEND=4,");    //Sende Messdaten

  Serial2.println(data.length() + 2);

  delay(50);
  if (Serial2.find('>')) {  //ESP8266 sendebereit
    Serial2.print(data);  //abschicken
    Serial2.print("\r\n");

  }

}
//---------------------------------------------------------------------------------

//Funktion UDP-Daten zur AiO (KBH2AiO) --------------------------------------------

void zurAiO(int wait)
{
  int tpos = 0;
  int tempPos = 0;
  String data;
  long int time = millis();
  while ( (time + wait) > millis())
  {

    while (Serial2.available())
    {
      char c = Serial2.read();
      if ((tpos < 500) && (c != '\n') && (c != '\r')) {
        antwort[tpos] = c;
        tpos++;
      }

    }
    antwort[tpos] = 0;
  }


  responce = String(antwort);

#ifdef dbg
  Serial.println(responce);
#endif

  if (modus == HAUPTMENUE)  // Wenn vorher Abbruch gesendet wurde
  {
    if (responce.indexOf("m27") > 0) //Starte Maischen
    {
      lcd.clear();
      modus = EINMAISCHEN;
      anfang = 0;
      signalton(500);
      meldung_detail = "Maischprogramm-gestartet!";
      return;
    }

    if (responce.indexOf("m3P") > 0) //Würzepumpe einschalten
    {
      lcd.clear();
      modus = WUERZEPUMPE;
      anfang = 0;
      signalton(500);
      meldung_detail = "Würzepumpe_aktiv";
      return;
    }

    if (responce.indexOf("wpv") > 0) //Würzepumpe einschalten
    {
      //     lcd.clear();
      splitmodus = "wpv";
      splitSetup();
      if (!uebertragungsfehler)
      {
        signalton(500);
        meldung_detail = "WPumpe_Verz_eing.";
      }
      return;
    }

    if (responce.indexOf("ng2") > 0) //Nachguss
    {
      lcd.clear();
      splitmodus = "ng2";
      splitSetup();
      if (!uebertragungsfehler)
      {
        signalton(500);
        meldung_detail = "Nachguss_wird_aufgeheizt!";
      }
      return;
    }

    if (responce.indexOf("m41") > 0) //Kochen
    {
      lcd.clear();
      modus = STARTKOCHEN;
      dclick = false;
      anfang = 0;
      heizung = EIN;
      signalton(500);
      meldung_detail = "Kochen-aufgerufen!";
      return;
    }

    if (responce.indexOf("mk1") > 0) //Kühlen
    {
      lcd.clear();
      splitmodus = "mk1";
      splitSetup();
      if (!uebertragungsfehler)
      {
        signalton(500);
        if (kuehlen) meldung_detail = "Kühlen-eingeschaltet";
        else meldung_detail = "Heizen-eingeschaltet";
        lcd.setCursor(0, 2); lcd.print("Modus = ");
        if (kuehlen) {
          lcd.print("K");
          lcd.print(char(0xF5));
          lcd.print("hlen");
          EEPROM.write(heiz_kuehl, 1);
        } else {
          lcd.print("Heizen");
          EEPROM.write(heiz_kuehl, 0);
        }
      }
      return;
    }

    if (responce.indexOf("mx236") > 0) //Einlesen Funksteckdosen
    {
      lcd.clear();
      modus = STECKDOSENCODE;
      anfang = 0;
      meldung_detail = "Funksteckdosencode-einlesen!";
      signalton(500);
      return;
    }


    if (responce.indexOf("RPT") > 0) {  //Rezept einlesen
      split();
      if (!uebertragungsfehler)
      {
        meldung_detail = "Rezept_erfolgreich_übertragen!";
        signalton(500);
      }
      return;
    }

    if (responce.indexOf("GFU") > 0) {  //Gärplan einlesen
      gaersplit();
      if (!uebertragungsfehler)
      {
        meldung_detail = "Gärplan_erfolgreich_übertragen!";
        signalton(500);
      }
      return;
    }
  }

  if (responce.indexOf("ms239") > 0) //Kochschwelle einstellen
  {
    splitmodus = "ms239";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Kochschwelle-eingestellt!";
      sollwert = kschwelle;
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("mt239") > 0) //Rührer/Pumpe konfigurieren
  {
    splitmodus = "mt239";
    splitSetup();
    if (!uebertragungsfehler)
    {
      signalton(500);
      meldung_detail = "Konfiguration-erfolgreich!";
    }
    return;
  }

  if (responce.indexOf("mr11") > 0) //Regler konfigurieren
  {
    splitmodus = "mr11";
    splitSetup();
    if (!uebertragungsfehler)
    {
      signalton(500);
      meldung_detail = "Regler-konfiguriert!";
    }
    return;
  }

  if (responce.indexOf("ms44") > 0) //Logging konfigurieren
  {
    splitmodus = "ms44";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Logging-konfiguriert!";
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("ms46") > 0) //Notfall
  {
    splitmodus = "ms46";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Notfallschaltung_aktiviert!";
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("ms236") > 0) //3. Funksteckdose
  {
    splitmodus = "ms236";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "3_Funksteckdose_konf.";
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("ms8") > 0) //Lautstärke Brauerruf
  {
#ifdef dbg
    Serial.println(responce);
#endif
    splitmodus = "ms8";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Brauerruf_Test";
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("mss8") > 0) //Lautstärke Brauerruf speichern
  {
#ifdef dbg
    Serial.println(responce);
#endif
    splitmodus = "mss8";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Brauerruf_eingestellt";
      signalton(500);
    }
    return;
  }

  if (responce.indexOf("msz") > 0) //Verlängerung Rastzeit
  {
#ifdef dbg
    Serial.println(responce);
#endif
    splitmodus = "msz";
    splitSetup();
    if (!uebertragungsfehler)
    {
      meldung_detail = "Rastzeit_verändert";
      signalton(500);
    }
    return;
  }



  if (responce.indexOf("m31") > 0) //Ruf quittieren
  {
    responce = "";
    meldung_detail = "Ruf-quittiert!";
    rufquittung = true;
    ruf = false;
    signalton(500);
    rufmodus = modus;
    modus = RUFALARM;
    if (altmodus == STARTKOCHEN) anfang = 3;
    if (altmodus == ENDTEMPERATURAUTOMATIK) modus = ABBRUCH;
    if (altmodus == NACHGUSS) modus = NACHGUSS;
    return;
  }

  if (responce.indexOf("m42") > 0) //Kochen Start
  {
    modus = KOCHEN;
    meldung_detail = "Kochen-gestartet";
    anfang = 0;
    signalton(500);
    return;
  }


  if (responce.indexOf("m80") > 0) //Abbruch
  {
    modus = ABBRUCH;
    anfang = 0;
    meldung_detail = "Abbruch";
    signalton(500);
    return;
  }

  if (responce.indexOf("m250") > 0) //Gärführung
  {
    modus = ANSTELLTEMPERATUR;
    anfang = 0;
    meldung_detail = "Gärführung-gestartet";
    signalton(500);
    return;
  }

  if (responce.indexOf("set") > 0) //Setup zur AiO
  {
    Setup_zum_pc();
    return;
  }

  if ((modus == HAUPTMENUE) && (pumpe))  //Pumpe im Setup ausgewählt?
  {
    if (responce.indexOf("mmazp") > 0) //Malzrohrpumpe Dauerbetrieb
    {
      signalton(500);
      if (!malzrohrpumpe_dauerbetrieb)
      {
        UmwaelzerEIN();
        malzrohrpumpe_dauerbetrieb = true;
        meldung_detail = "Malzrohrpumpe-Dauerbetrieb";
      } else
      {
        UmwaelzerAUS();
        malzrohrpumpe_dauerbetrieb = false;
        meldung_detail = "Malzrohrpumpe-Dauerbetrieb-AUS";
      }
    }
  }

#ifdef geschwindigkeitsrelais
  if ((modus == MAISCHEN_START) || (modus == EINMAISCHEN) || (modus == TEMPERATURAUTOMATIK) || (modus == ZEITAUTOMATIK) || (modus == ENDTEMPERATURAUTOMATIK))
    if (responce.indexOf("spe") > 0) //Rührer Geschwindigkeit ändern
      if (ruehrer)
        //       if (geschwindigkeitsrelais)
      {
        meldung = "Rührer_Geschwindigkeit";
        if (highspeed) {
          digitalWrite(Speedrelais, LOW_SPEED);
          meldung_detail = "Rührer_Low_Speed";
        } else {
          digitalWrite(Speedrelais, HIGH_SPEED);
          meldung_detail = "Rührer_High_Speed";
        }
        signalton(500);
        highspeed = !highspeed;
      }
      else meldung_detail = "Speedrelais_inaktiv";
#endif
}
//---------------------------------------------------------------------------------

//Funktion Setupdaten aufsplitten -------------------------------------------------
void splitSetup()
{ byte n;
  byte dummy;
  String dummystring;
  char * pch;
  pch = strtok (antwort, ";:");
  n = 0;
  uebertragungsfehler = false;

  if (splitmodus == "ms239")  //Kochschwelle
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        kschwelle = atoi(pch);
        if (EEPROM.read(Kochschwelle) != kschwelle) EEPROM.write(Kochschwelle, kschwelle);
      }
      n++;
    }
    if (n != 3) uebertragungsfehler = true;
  }

  if (splitmodus == "ms46")  //Notfallmodus
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        if (EEPROM.read(notfall) != atoi(pch)) EEPROM.write(notfall, atoi(pch));
        notfall_aktiv = atoi(pch);
      }
      n++;
    }
    if (n != 3) uebertragungsfehler = true;
  }

  if (splitmodus == "ms236")
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        if (EEPROM.read(dritte_funksteckdose) != atoi(pch)) EEPROM.write(dritte_funksteckdose, atoi(pch));
        if (atoi(pch) == 1) {
          funkruf = true;
        } else {
          funkruf = false;
        }
      }
      n++;
    }

    if (n != 3) uebertragungsfehler = true;
  }


  if (splitmodus == "ms8")   //Lautstärke testen
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) drehen = atoi(pch);
      n++;
    }
    if (n != 3) uebertragungsfehler = true; else {
      modus = SOUNDEINSTELLUNG;
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ton: ");
    }
  }

  if (splitmodus == "msz")   //Rastzeit verändern
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        if (modus == ZEITAUTOMATIK) drehen = atoi(pch);
        if (modus == GAERZEITAUTOMATIK) drehen = atoi(pch);
      }
      n++;
    }
    if (n != 3) uebertragungsfehler = true;
  }

  if (splitmodus == "wpv")   //Einschaltverzögerung Würzepumpe
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) esvpumpen = atoi(pch) * 1000;
        if (n == 2) if (atoi(pch) == 1) manuell_ein = false; else manuell_ein = true;
#ifdef dbg
      if (manuell_ein) Serial.println("Manuell"); else Serial.println("Automatik");
#endif
      n++;
    }

    if (n != 4) uebertragungsfehler = true;
  }

  if (splitmodus == "mss8")  //Lautstärke speichern
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) drehen = atoi(pch);
      n++;
    }
    if (n != 3) uebertragungsfehler = true; else {
      if (EEPROM.read(sound) != drehen) EEPROM.write(sound, drehen);
//      B_EIN = drehen;
      modus = ABBRUCH;
    }
  }


  if (splitmodus == "mk1")  //Heizen/Kühlen umschalten
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        sollwert = atoi(pch);
        drehen = sollwert;
        modus = KUEHLEN;
      }

      if (n == 2) {
        if (atoi(pch) == 1) kuehlen = true; else kuehlen = false;
      }

      n++;
    }
    if (n != 4) uebertragungsfehler = true;
  }

  if (splitmodus == "ng2")  //Nachguss
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        sollwert = atoi(pch);
        nachgusssollwert = sollwert;
        modus = NACHGUSS;
        anfang = 0;
      }
      n++;
    }

    if (n != 3) uebertragungsfehler = true;
  }

  if (splitmodus == "mt239")  //Mischerwahl
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1)
      { dummy = atoi(pch); if (EEPROM.read(umwaelzwahl) != dummy) EEPROM.write(umwaelzwahl, dummy);
        switch (dummy)
        {
          case 0 : {
              ruehrer = true;
              pumpe = false;
              break;
            }
          case 1 : {
              ruehrer = false;
              pumpe = true;
              break;
            }
        }
      }

      if (n == 2)
      {

        if (String(pch) == "N") {
          if (EEPROM.read(udauer) != 0) {
            EEPROM.write(udauer, 0);
            dauerbetrieb = 'N';
          }
        }
        if (String(pch) == "J") {
          if (EEPROM.read(udauer) != 1) {
            EEPROM.write(udauer, 1);
            dauerbetrieb = 'J';
          }
        }

      }

      if (n == 3)
      {
        dummy = atoi(pch);
        if (EEPROM.read(umwaelzer_ie) != dummy) EEPROM.write(umwaelzer_ie, dummy);
        intervalEin = dummy;
      }

      if (n == 4)
      {
        dummy = atoi(pch);
        if (EEPROM.read(umwaelzer_ia) != dummy) EEPROM.write(umwaelzer_ia, dummy);
        intervalAus = dummy;
      }

      if (n == 5)
      {
        dummy = atoi(pch);
        if (EEPROM.read(ruepumeinmaisch) != dummy) EEPROM.write(ruepumeinmaisch, dummy);
        ruehrer_beim_einmaischen = dummy;
      }

      n++;
    }

    if (n != 7) uebertragungsfehler = true;
  }

  if (splitmodus == "mr11")  //Regler konfigurieren
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      dummystring = pch;

      if (n == 1) {
        if (EEPROMReadlong(graf_h) != dummystring.toFloat() * 10) EEPROMWritelong(graf_h,  dummystring.toFloat() * 10);
        kh = EEPROMReadlong(graf_h) / 10;
      }

      if (n == 2) {
        if (EEPROMReadlong(wein_h) != dummystring.toFloat() * 1000L) EEPROMWritelong(wein_h, dummystring.toFloat() * 1000L);
        esvheizen = EEPROMReadlong(wein_h);
      }

      if (n == 3) {
        if (EEPROMReadlong(graf_k) != dummystring.toFloat() * 10) EEPROMWritelong(graf_k, dummystring.toFloat() * 10);
        kk = EEPROMReadlong(graf_k) / 10;
      }

      if (n == 4) {
        if (EEPROMReadlong(kuehlverz) != dummystring.toFloat() * 60000L) EEPROMWritelong(kuehlverz, dummystring.toFloat() * 60000L);;
        kzeitverz = EEPROMReadlong(kuehlverz) * 60000L;
      }

      n++;
    }
    if (n != 6) uebertragungsfehler = true;
  }

  if (splitmodus == "ms44")  //Loggerwahl
  {
    while (pch != NULL)
    {
      pch = strtok (NULL, ";");
      if (n == 1) {
        if (atoi(pch) == 3)
        {
          if (EEPROM.read(logging) != 0) EEPROM.write(logging, 0);
        }
        else
        {
          if (EEPROM.read(logging) != 1) EEPROM.write(logging, 1);
          if (EEPROM.read(loggerwahl) != atoi(pch)) EEPROM.write(loggerwahl, atoi(pch));
          drehen = EEPROM.read(loggerwahl);
          switch (drehen)
          {
            case 0 : {
                serialcom = true;
                logview = false;
                break;
              }
            case 1 : {
                logview = true;
                serialcom = false;
                break;
              }
          }
        }
      }
      n++;
    }
    if (n != 3) uebertragungsfehler = true;
  }
}
//---------------------------------------------------------------------------------

//Funktion Rezeptdaten aufsplitten ------------------------------------------------

void split()
{ byte n;
  char * pch;
  pch = strtok (antwort, ";:");
  n = 0;
  uebertragungsfehler = false;
  while (pch != NULL)
  {
    pch = strtok (NULL, ";");
    if (n == 1) maischtemp = atoi(pch);
    if (n == 2) rasten = atoi(pch);
    if (n == 3) kochzeit = atoi(pch);
    if (n == 4) rastTemp[1] = atoi(pch);
    if (n == 5) rastTemp[2] = atoi(pch);
    if (n == 6) rastTemp[3] = atoi(pch);
    if (n == 7) rastTemp[4] = atoi(pch);
    if (n == 8) rastTemp[5] = atoi(pch);
    if (n == 9) rastTemp[6] = atoi(pch);
    if (n == 10) rastTemp[7] = atoi(pch);
    if (rasten == 1) endtemp = rastTemp[2];
    if (rasten == 2) endtemp = rastTemp[3];
    if (rasten == 3) endtemp = rastTemp[4];
    if (rasten == 4) endtemp = rastTemp[5];
    if (rasten == 5) endtemp = rastTemp[6];
    if (rasten == 6) endtemp = rastTemp[7];


    if (n == 11) rastZeit[1] = atoi(pch);
    if (n == 12) rastZeit[2] = atoi(pch);
    if (n == 13) rastZeit[3] = atoi(pch);
    if (n == 14) rastZeit[4] = atoi(pch);
    if (n == 15) rastZeit[5] = atoi(pch);
    if (n == 16) rastZeit[6] = atoi(pch);
    if (n == 17) rastZeit[7] = atoi(pch);
    if (n == 18)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[0] = atoi(pch);
    if (n == 19)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[1] = atoi(pch);
    if (n == 20)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[2] = atoi(pch);
    if (n == 21)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[3] = atoi(pch);
    if (n == 22)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[4] = atoi(pch);
    if (n == 23)
        if (atoi(pch) < 0) nachisomerisierung = abs(atoi(pch)); else hopfengabe[5] = atoi(pch);

    if (n == 24) hanz = atoi(pch);
    if (n == 25) nachisomerisierung = atoi(pch);
    n++;
  }
  rezept_eingelesen = true;
  if (n != 27) uebertragungsfehler = true;

}
//---------------------------------------------------------------------------------

//Funktion Gärplan aufsplitten ------------------------------------------------

void gaersplit()
{ byte n;
  char * pch;
  pch = strtok (antwort, ";:");
  n = 0;
  uebertragungsfehler = false;
  while (pch != NULL)
  {
    pch = strtok (NULL, ";");
    if (n == 1) gaerrasten = atoi(pch);
    if (n == 2) gaerRastTemp[1] = atoi(pch);
    if (n == 3) gaerRastTemp[2] = atoi(pch);
    if (n == 4) gaerRastTemp[3] = atoi(pch);
    if (n == 5) gaerRastTemp[4] = atoi(pch);
    if (n == 6) gaerRastTemp[5] = atoi(pch);

    if (n == 7) gaerRastZeit[1] = atoi(pch);
    if (n == 8) gaerRastZeit[2] = atoi(pch);
    if (n == 9) gaerRastZeit[3] = atoi(pch);
    if (n == 10) gaerRastZeit[4] = atoi(pch);
    if (n == 11) gaerRastZeit[5] = atoi(pch);
    if (n == 12) gaerstart = atoi(pch);

    n++;
  }

  if (n != 14) uebertragungsfehler = true;
}
//---------------------------------------------------------------------------------

//Funktion Setupdaten der AiO zum PC ----------------------------------------------

void Setup_zum_pc()
{
  String data;
  data = String(kschwelle);  //Kochschwelle
  data += ";";
  if (EEPROM.read(udauer) == 0) dauerbetrieb = 'N';
  if (EEPROM.read(udauer) == 1) dauerbetrieb = 'J';
  data += dauerbetrieb;  //Rührer Dauerbetrieb?
  data += ";";

  intervalEin = EEPROM.read(umwaelzer_ie);
  intervalAus = EEPROM.read(umwaelzer_ia);

  data += String( intervalEin);  //Rührer/Malzrohrpumpe EIN Intervall
  data += ";";
  data += String( intervalAus);  //Rührer/Malzrohrpumpe AUS Intervall
  data += ";";
  byte wahl = EEPROM.read(umwaelzwahl);

  data += String(wahl);  //Rührer oder Malzrohrpumpe

  data += ";";
  data += String(esvheizen);  //Einschaltverzögerung Heizen
  data += ";";

  data += String(EEPROMReadlong(kuehlverz) / 60000L); //Einschaltverzögerung Kühlen
  data += ";";
  data += String(EEPROMReadlong(graf_h));  //Gradientenfaktor Heizen
  data += ";";
  data += String(EEPROMReadlong(graf_k)); //Gradientenfaktor Kühlen
  data += ";";
  data += String(EEPROM.read(logging));  //Logging aktiv?
  data += ";";
  data += String(EEPROM.read(loggerwahl));  //Welches Loggingprogramm aktiv?
  data += ";";
  data += String(notfall_aktiv);  //Stromausfallmodus (Notfall) aktiv?
  data += ";";
  if (modus == NACHGUSS) data += String(drehen); else data += "78";  //Nachgusstemperatur
  data += ";";
  wahl = EEPROM.read(dritte_funksteckdose);
  data += String(wahl);  //Ruf oder Nachgussheizung
  data += ";";
  wahl = EEPROM.read(sound);
  data += String(wahl);  //Lautstärke
  data += ";";
  wahl = EEPROM.read(ruepumeinmaisch);
  data += String(wahl);  //Mischer beim Einmaischen aktiv?
  data += ";";
  wahl = esvpumpen / 1000;
  data += String(wahl);  //Einschaltverzögerung Würzepumpe im Automatikbetrieb
  data += ";";
  if (manuell_ein) wahl = 0; else wahl = 1;
  data += String(wahl);  //Würzepumpe Manuell/Automatikbetrieb
  data += ";";




#ifdef dbg
  Serial.println(data); //Kontrolle
#endif
  Serial2.print("AT+CIPSEND=4,");    //Sende Messdaten

  Serial2.println(data.length() + 2);

  delay(50);
  if (Serial2.find('>')) {  //ESP8266 sendebereit
    Serial2.print(data);  //abschicken
    Serial2.print("\r\n");
  }
}

void HeizungAUS()
{
  digitalWrite(HeizungR1, H_AUS);  //Heizung aus
  digitalWrite(HeizungR2, H_AUS);
  mySwitch.send(funk_heizung_aus, 24);
  firstinterval = true;
  delay(100);
  if ((modus == MAISCHEN_START) || (modus == EINMAISCHEN) || (modus == TEMPERATURAUTOMATIK) || (modus == ZEITAUTOMATIK) || (modus == ENDTEMPERATURAUTOMATIK)) {
    rwerk = 0;
    umwaelzerEinAus(1);
  }
}

void HeizungEIN()
{
  digitalWrite(HeizungR1, H_EIN);   // einschalten
  digitalWrite(HeizungR2, H_EIN);   // einschalten
  mySwitch.send(funk_heizung_ein, 24); //nicht bei jedem Durchlauf aufrufen
  delay(100);
}

void NachgussAUS()
{
  digitalWrite(NachgussR1, NG_AUS);  //Nachgussrelais aus
  digitalWrite(NachgussR2, NG_AUS);
  if (!funkruf) {
    mySwitch.send(funk_variabel_aus, 24);
    delay(100);
  }
  nachgussheizung = AUS;
}

void NachgussEIN()
{
  if (!funkruf) {
    mySwitch.send(funk_variabel_ein, 24);
    delay(100);
  }
  digitalWrite(NachgussR1, NG_EIN);   // Nachgussrelais einschalten
  digitalWrite(NachgussR2, NG_EIN);   // Nachgussrelais einschalten
}

void UmwaelzerAUS()
{
  digitalWrite(UmwaelzR1, R_AUS);     //Mischer (Umwälzer) AUS
  digitalWrite(UmwaelzR2, R_AUS);
  mySwitch.send(funk_umwaelzer_aus, 24);  //24 Bit -Länge - füllt den Wert mit 0en
  delay(100);
}

void UmwaelzerEIN()
{
  digitalWrite(UmwaelzR2, R_EIN);
  digitalWrite(UmwaelzR1, R_EIN);
  mySwitch.send(funk_umwaelzer_ein, 24);
  delay(100);
}

void WuerzepumpeEinAus(int EinAus)
{
  if (EinAus == 0) digitalWrite(fuellstand_pumpe, WUERZEPUMPE_AUS);  //Pumpe ist ausgeschaltet
  else digitalWrite(fuellstand_pumpe, WUERZEPUMPE_EIN);
}

void RufEIN()
{
  digitalWrite(RufR, B_EIN);   // Ruf einschalten
  if (funkruf) mySwitch.send(funk_variabel_ein, 24);
  delay(100);
}

void RufAUS()
{
  digitalWrite(RufR, B_AUS);   // Ruf ausschalten
  if (funkruf) mySwitch.send(funk_variabel_aus, 24);
  delay(100);
}

//-------------------------------------------------------------------------------------
//Ende des Programms
//-------------------------------------------------------------------------------------



